# -*- coding: utf-8 -*-

from imio.smartweb.core import config
from imio.smartweb.core.tests.utils import get_json
from plone import api
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    applyProfile,
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
)
from plone.testing.zope import WSGI_SERVER_FIXTURE
from zope.component import getUtility
from zope.globalrequest import setRequest
from zope.schema.interfaces import IVocabularyFactory

import imio.smartweb.core
import imio.smartweb.core.contents.pages.footer.content as footer_content
import imio.smartweb.core.contents.pages.herobanner.content as herobanner_content
import mock
import json
import requests_mock
import unittest


class ImioSmartwebCoreLayer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.restapi

        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=imio.smartweb.core, name="testing.zcml")

    @requests_mock.Mocker()
    def setUpPloneSite(self, portal, m):
        request = portal.REQUEST
        setRequest(request)
        json_directory_entities_raw_mock = get_json(
            "resources/json_directory_entities_raw_mock.json"
        )
        m.get(
            f"{config.DIRECTORY_URL}/@search?portal_type=imio.directory.Entity&sort_on=sortable_title&b_size=1000000&metadata_fields=UID",
            text=json.dumps(json_directory_entities_raw_mock),
        )
        json_events_entities_raw_mock = get_json(
            "resources/json_events_entities_raw_mock.json"
        )
        m.get(
            f"{config.EVENTS_URL}/@search?portal_type=imio.events.Entity&sort_on=sortable_title&b_size=1000000&metadata_fields=UID",
            text=json.dumps(json_events_entities_raw_mock),
        )
        json_news_entities_raw_mock = get_json(
            "resources/json_news_entities_raw_mock.json"
        )
        m.get(
            f"{config.NEWS_URL}/@search?portal_type=imio.news.Entity&sort_on=sortable_title&b_size=1000000&metadata_fields=UID",
            text=json.dumps(json_news_entities_raw_mock),
        )
        applyProfile(portal, "imio.smartweb.core:testing")
        api.user.create(email="test@imio.be", username="test")
        api.user.grant_roles(username="test", roles=["Site Administrator"])
        footer_content.ban_physicalpath = mock.Mock(return_value=None)
        herobanner_content.ban_physicalpath = mock.Mock(return_value=None)


IMIO_SMARTWEB_CORE_FIXTURE = ImioSmartwebCoreLayer()


IMIO_SMARTWEB_CORE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(IMIO_SMARTWEB_CORE_FIXTURE,),
    name="ImioSmartwebCoreLayer:IntegrationTesting",
)


IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(IMIO_SMARTWEB_CORE_FIXTURE,),
    name="ImioSmartwebCoreLayer:FunctionalTesting",
)


IMIO_SMARTWEB_CORE_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        IMIO_SMARTWEB_CORE_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        WSGI_SERVER_FIXTURE,
    ),
    name="ImioSmartwebCoreLayer:AcceptanceTesting",
)


class ImioSmartwebTestCase(unittest.TestCase):
    def assertVocabularyLen(self, vocname, voc_len):
        factory = getUtility(IVocabularyFactory, vocname)
        vocabulary = factory()
        self.assertEqual(len(vocabulary), voc_len)



# -*- coding: utf-8 -*-

from datetime import date, datetime
from datetime import timedelta
from decimal import Decimal
from imio.smartweb.common.utils import rich_description
from imio.smartweb.core.utils import batch_results
from imio.smartweb.core.utils import get_json
from imio.smartweb.core.utils import hash_md5
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from zope.i18n import translate
from zope.i18nmessageid import MessageFactory

import json


class ContactProperties:
    def __init__(self, json_dict):
        self.contact = json_dict

    def __getattr__(self, name):
        return self.contact.get(name)

    @property
    def contact_type_class(self):
        contact_type = self.contact.get("type").get("token")
        return "contact-type-{}".format(contact_type)

    @property
    def description(self):
        description = rich_description(self.contact.get("description"))
        return description

    def logo(self):
        if self.contact.get("logo") is None:
            return ""
        modified_hash = hash_md5(self.contact["modified"])
        logo = f"{self.contact['@id']}/@@images/logo/medium?cache_key={modified_hash}"
        return logo

    def leadimage(self):
        if self.contact.get("image") is None:
            return ""
        modified_hash = hash_md5(self.contact["modified"])
        logo = f"{self.contact['@id']}/@@images/image/medium?cache_key={modified_hash}"
        return logo

    def data_geojson(self):
        """Return the contact geolocation as GeoJSON string."""
        current_lang = api.portal.get_current_language()[:2]
        coordinates = self.contact.get("geolocation")
        longitude = coordinates.get("longitude")
        latitude = coordinates.get("latitude")
        link_text = translate(_("Itinerary"), target_language=current_lang)
        geo_json = {
            "type": "Feature",
            "properties": {
                "popup": '<a href="{}">{}</a>'.format(
                    self.get_itinerary_link(), link_text
                ),
            },
            "geometry": {
                "type": "Point",
                "coordinates": [
                    longitude,
                    latitude,
                ],
            },
        }
        return json.dumps(geo_json)

    def images(self, image_scale, nb_results_by_batch):
        contact_url = self.contact["@id"]
        query = "@search?portal_type=Image&path.depth=1&metadata_fields=modified"
        images_url_request = "{}/{}".format(contact_url, query)
        json_images = get_json(images_url_request)
        if json_images is None or len(json_images.get("items", [])) == 0:
            return
        results = []
        thumb_scale = image_scale
        for image in json_images.get("items"):
            base_url = image["@id"]
            modified_hash = hash_md5(image["modified"])
            large_url = (
                f"{base_url}/@@images/image/extralarge?cache_key={modified_hash}"
            )
            url = f"{base_url}/@@images/image/{thumb_scale}?cache_key={modified_hash}"
            dict_item = {
                "title": image["title"],
                "description": image["description"],
                "image_large_url": large_url,
                "image_url": url,
            }
            results.append(dict_item)
        return batch_results(results, nb_results_by_batch)

    @property
    def days(self):
        return (
            ("monday", _("Monday")),
            ("tuesday", _("Tuesday")),
            ("wednesday", _("Wednesday")),
            ("thursday", _("Thursday")),
            ("friday", _("Friday")),
            ("saturday", _("Saturday")),
            ("sunday", _("Sunday")),
        )

    def get_itinerary_link(self):
        address_parts = [
            self.contact.get("street"),
            self.contact.get("number") and str(self.contact.get("number")) or "",
            self.contact.get("complement"),
            self.contact.get("zipcode") and str(self.contact.get("zipcode")) or "",
            self.contact.get("city"),
        ]
        if self.contact.get("country"):
            address_parts.append(self.contact.get("country").get("title"))
        address = "+".join(filter(None, address_parts))
        if not address:
            return
        return "https://www.google.com/maps/dir/?api=1&destination={}".format(address)

    def get_opening_informations(self, a_date=None):
        current_date = a_date or date.today()
        schedule = self.contact.get("schedule")
        multi_schedule = self.contact.get("multi_schedule")
        exceptional_closure = self.contact.get("exceptional_closure")
        week_day = self.days[current_date.weekday()][0]
        day_schedule = schedule[week_day]
        for schedule_info in multi_schedule:
            if schedule_info.get("dates") == []:
                continue
            for daterange in schedule_info.get("dates"):
                min_date = datetime.strptime(
                    daterange.get("start_date"), "%Y-%m-%d"
                ).date()
                max_date = datetime.strptime(
                    daterange.get("end_date"), "%Y-%m-%d"
                ).date()
                if not (min_date < current_date < max_date):
                    week_day = week_day  # Fix coverage issue with continue.
                    continue
                day_schedule = schedule_info.get("schedule")[week_day]

        for exceptional_closure_info in exceptional_closure:
            exceptional_closure_date = datetime.strptime(
                exceptional_closure_info.get("date"), "%Y-%m-%d"
            ).date()
            if not current_date == exceptional_closure_date:
                continue
            for k, v in day_schedule.items():
                day_schedule[k] = ""
                if k == "comment":
                    day_schedule[k] = exceptional_closure_info.get("title")
        # sample : {'afternoonend': '16:00', 'afternoonstart': '14:00', 'comment': '', 'morningend': '14:00', 'morningstart': '09:00'}
        return day_schedule

    def get_week_days(self):
        """Returns a list of Messages for the weekday names."""
        PLMF = MessageFactory("plonelocales")
        ts = api.portal.get_tool("translation_service")
        weekdays = []
        now = date.today()
        for i in range(0, 7):
            ref_date = now + timedelta(days=i)
            # weekday 0 == monday in python
            # ts.day_msgid 0 = sunday in plone
            ref_date_tsweekday = (ref_date.weekday() + 1) % 7
            weekdays.append(
                {
                    PLMF(
                        ts.day_msgid(ref_date_tsweekday, format="s"),
                        default=ts.weekday_english(ref_date_tsweekday, format="a"),
                    ): self.get_schedule_for_date(ref_date)
                }
            )
        return weekdays

    def get_schedule_for_date(self, a_date):
        return self.get_opening_informations(a_date)

    def get_schedule_for_today(self, schedule):
        return get_schedule_for_today(schedule)

    def formatted_schedule(self, schedule):
        return formatted_schedule(schedule)

    def is_empty_schedule(self):
        schedule = self.contact.get("schedule") or {}
        for day in schedule.values():
            for value in day.values():
                if value:
                    return False
        return True

    def formatted_address(self):
        street_parts = [
            self.contact.get("street"),
            self.contact.get("number") and str(self.contact.get("number")) or "",
            self.contact.get("complement"),
        ]
        street = " ".join(filter(None, street_parts))
        entity_parts = [
            self.contact.get("zipcode") and str(self.contact.get("zipcode")) or "",
            self.contact.get("city"),
        ]
        entity = " ".join(filter(None, entity_parts))
        country = (
            self.contact.get("country")
            and self.contact.get("country").get("title")
            or ""
        )
        if not (street or entity or country):
            return None
        return {"street": street, "entity": entity, "country": country}


def formatted_schedule(schedule):
    # opening = {'afternoonend': '', 'afternoonstart': '', 'comment': '', 'morningend': '12:00', 'morningstart': '08:30'}
    current_lang = api.portal.get_current_language()[:2]
    morningstart = schedule.get("morningstart")
    morningend = schedule.get("morningend")
    afternoonstart = schedule.get("afternoonstart")
    afternoonend = schedule.get("afternoonend")
    comment = schedule.get("comment")
    str_opening = ""
    morning = False
    if morningend:
        morning = True
        str_opening = "{} - {}".format(morningstart, morningend)
    if morning is True and afternoonstart:
        str_opening += " | {} - {}".format(afternoonstart, afternoonend)
    elif morning is False and afternoonstart:
        str_opening = "{} - {}".format(afternoonstart, afternoonend)
    elif not morningend and not afternoonstart:
        str_opening = "{} - {}".format(morningstart, afternoonend)
    if not morningstart and not morningend and not afternoonstart and not afternoonend:
        str_opening = translate(_("Closed"), target_language=current_lang)
    return "{} ({})".format(str_opening, comment) if comment else str_opening


# {'afternoonend': '', 'afternoonstart': '', 'comment': 'vendredi : apéro à midi', 'morningend': '11:00', 'morningstart': '08:30'}
def get_schedule_for_today(schedule):
    current_lang = api.portal.get_current_language()[:2]
    message = translate(_("Open"), target_language=current_lang)
    comment = schedule.get("comment") or ""
    morningstart = Decimal(schedule.get("morningstart").replace(":", ".") or "0.00")
    morningend = Decimal(schedule.get("morningend").replace(":", ".") or "0.00")
    afternoonstart = Decimal(schedule.get("afternoonstart").replace(":", ".") or "0.00")
    afternoonend = Decimal(schedule.get("afternoonend").replace(":", ".") or "0.00")

    now_str = Decimal(datetime.now().strftime("%H.%M"))
    if not morningstart and not morningend and not afternoonstart and not afternoonend:
        translated_closed = translate(_("Closed"), target_language=current_lang)
        return (
            "{} ({})".format(translated_closed, comment)
            if comment
            else translated_closed
        )
    before_opened = now_str < (morningstart or afternoonstart)
    if before_opened:
        translated_open_at = translate(_("Open at "), target_language=current_lang)
        hour, minute = str(morningstart or afternoonstart).split(".")
        message = "{} {:02d}:{:02d}".format(translated_open_at, int(hour), int(minute))

    lunch_time = is_now_between(now_str, morningend, afternoonstart)
    if lunch_time:
        message = translate(_("Lunch time"), target_language=current_lang)

    after_closed = now_str >= (afternoonend or morningend)
    if after_closed:
        message = translate(_("Closed"), target_language=current_lang)
    return "{} ({})".format(message, comment) if comment else message


def is_now_between(now, start, end):
    if not start or not end:
        return False
    return now >= start and now < end



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform import directives
from plone.dexterity.content import Container
from plone.supermodel import model
from zope import schema
from zope.interface import implementer


class IDirectoryView(model.Schema):
    """Marker interface and Dexterity Python Schema for DirectoryView"""

    directives.widget(selected_categories=SelectFieldWidget)
    selected_categories = schema.List(
        title=_("Selected Categories"),
        value_type=schema.Choice(
            vocabulary="imio.smartweb.vocabulary.DirectoryCategories"
        ),
        required=False,
    )

    nb_results = schema.Int(
        title=_("Number of items to display"), default=20, required=True
    )


@implementer(IDirectoryView)
class DirectoryView(Container):
    """DirectoryView class"""



# -*- coding: utf-8 -*-

from freezegun import freeze_time
from imio.smartweb.common.utils import get_vocabulary
from imio.smartweb.core import config
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_json
from plone import api
from unittest.mock import patch

import json
import requests
import requests_mock

GUICHET_URL = "https://demo.guichet-citoyen.be/api/formdefs/"


class TestVocabularies(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.json_procedures_raw_mock = get_json(
            "resources/json_procedures_raw_mock.json"
        )
        self.json_contacts_raw_mock = get_json("resources/json_contacts_raw_mock.json")
        self.json_empty_contacts_raw_mock = get_json(
            "resources/json_no_contact_raw_mock.json"
        )
        self.contact_search_url = "http://localhost:8080/Plone/@search?portal_type=imio.directory.Contact&selected_entities=396907b3b1b04a97896b12cc792c77f8&sort_on=breadcrumb&b_size=1000000&metadata_fields=UID&metadata_fields=breadcrumb"

    def test_icons(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.Icons", 49)
        vocabulary = get_vocabulary("imio.smartweb.vocabulary.Icons")
        term = vocabulary.getTerm("action.email")
        self.assertEqual(term.title, "Action - Email")

    @requests_mock.Mocker()
    def test_procedure_keys(self, m):
        m.get(GUICHET_URL, text=json.dumps(self.json_procedures_raw_mock))
        self.assertVocabularyLen("imio.smartweb.vocabulary.PublikProcedures", 3)
        vocabulary = get_vocabulary("imio.smartweb.vocabulary.PublikProcedures")
        self.assertEqual(
            vocabulary.getTerm(
                "https://demo-formulaires.guichet-citoyen.be/acte-de-deces/"
            ).title,
            "Acte de d\\u00e9c\\u00e8s",
        )
        self.assertEqual(
            vocabulary.getTerm(
                "https://demo-formulaires.guichet-citoyen.be/acte-de-divorce/"
            ).title,
            "Acte de divorce",
        )
        self.assertEqual(
            vocabulary.getTerm(
                "https://demo-formulaires.guichet-citoyen.be/acte-de-mariage/"
            ).title,
            "Acte de mariage",
        )
        api.portal.set_registry_record(
            "smartweb.url_formdefs_api",
            "https://demo.guichet-citoyen.be/api/formdefs/?query=kamoulox",
        )
        m.get(
            "https://demo.guichet-citoyen.be/api/formdefs/?query=kamoulox",
            text=json.dumps(self.json_procedures_raw_mock),
        )
        self.assertVocabularyLen("imio.smartweb.vocabulary.PublikProcedures", 3)

    @requests_mock.Mocker()
    def test_procedure_error_values(self, m):
        m.get(GUICHET_URL, exc=requests.exceptions.ConnectTimeout)
        self.assertVocabularyLen("imio.smartweb.vocabulary.PublikProcedures", 0)

        m.get(GUICHET_URL, status_code=404)
        self.assertVocabularyLen("imio.smartweb.vocabulary.PublikProcedures", 0)

        api.portal.set_registry_record("smartweb.url_formdefs_api", "")
        m.get(GUICHET_URL, text=json.dumps(self.json_procedures_raw_mock))
        self.assertVocabularyLen("imio.smartweb.vocabulary.PublikProcedures", 0)

    def test_bootstrap_css(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.BootstrapCSS", 6)

    def test_subsite_display_mode(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.SubsiteDisplayMode", 3)

    def test_contact_blocks(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.ContactBlocks", 10)

    @requests_mock.Mocker()
    def test_empty_remote_directory_entities(self, m):
        m.get(
            self.contact_search_url, text=json.dumps(self.json_empty_contacts_raw_mock)
        )
        self.assertVocabularyLen("imio.smartweb.vocabulary.RemoteDirectoryEntities", 0)

    @requests_mock.Mocker()
    def test_remote_directory_entities(self, m):
        json_entities_raw_mock = get_json(
            "resources/json_directory_entities_raw_mock.json"
        )
        url = "{}/@search?portal_type=imio.directory.Entity&sort_on=sortable_title&b_size=1000000&metadata_fields=UID".format(
            config.DIRECTORY_URL
        )
        m.get(url, text=json.dumps(json_entities_raw_mock))
        self.assertVocabularyLen("imio.smartweb.vocabulary.RemoteDirectoryEntities", 2)
        vocabulary = get_vocabulary("imio.smartweb.vocabulary.RemoteDirectoryEntities")
        self.assertEqual(
            vocabulary.getTerm("7c69f9a738ec497c819725c55888ee3a").title,
            "Belleville",
        )

    @requests_mock.Mocker()
    def test_remote_agendas(self, m):
        json_entities_raw_mock = get_json(
            "resources/json_events_entities_raw_mock.json"
        )
        url = f"{config.EVENTS_URL}/@search?UID=7c69f9a738ec497c819725c55888ee31"
        m.get(url, text=json.dumps(json_entities_raw_mock))
        json_agendas_raw_mock = get_json("resources/json_events_agendas_raw_mock.json")
        url = f"{config.EVENTS_URL}/imio-events-entity/@search?portal_type=imio.events.Agenda&sort_on=sortable_title&b_size=1000000&metadata_fields=UID"
        m.get(url, text=json.dumps(json_agendas_raw_mock))
        self.assertVocabularyLen("imio.smartweb.vocabulary.RemoteAgendas", 2)
        vocabulary = get_vocabulary("imio.smartweb.vocabulary.RemoteAgendas")
        self.assertEqual(
            vocabulary.getTerm("64f4cbee9a394a018a951f6d94452914").title,
            "Agenda administration de Belleville",
        )

    @requests_mock.Mocker()
    def test_remote_news_empty_entity(self, m):
        # mock "control panel" to set entity
        url = "http://localhost:8080/Plone/@search?metadata_fields=UID&portal_type=imio.news.Entity"
        json_entities_raw_mock = get_json("resources/json_news_entities_raw_mock.json")
        m.get(url, text=json.dumps(json_entities_raw_mock))
        api.portal.set_registry_record(
            "smartweb.news_entity_uid", "zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz"
        )

        # mock 2 : search one entity
        json_entity_raw_mock = get_json("resources/json_search_one_news_entity.json")
        url = f"{config.NEWS_URL}/@search?UID=zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz"
        m.get(url, text=json.dumps(json_entity_raw_mock))

        # mock 3
        json_no_newsfolder_raw_mock = get_json(
            "resources/json_no_newsfolder_raw_mock.json"
        )
        url = "http://localhost:8080/Plone/imio-news-empty-entity/@search?portal_type=imio.news.NewsFolder&sort_on=sortable_title&b_size=1000000&metadata_fields=UID"
        m.get(url, text=json.dumps(json_no_newsfolder_raw_mock))
        self.assertVocabularyLen("imio.smartweb.vocabulary.RemoteNewsFolders", 0)

    @requests_mock.Mocker()
    def test_remote_newsfolder(self, m):
        json_entities_raw_mock = get_json("resources/json_news_entities_raw_mock.json")
        url = f"{config.NEWS_URL}/@search?UID=7c69f9a738ec497c819725c55888ee32"
        m.get(url, text=json.dumps(json_entities_raw_mock))
        json_newsfolder_raw_mock = get_json(
            "resources/json_news_newsfolder_raw_mock.json"
        )
        url = f"{config.NEWS_URL}/imio-news-entity/@search?portal_type=imio.news.NewsFolder&sort_on=sortable_title&b_size=1000000&metadata_fields=UID"
        m.get(url, text=json.dumps(json_newsfolder_raw_mock))
        self.assertVocabularyLen("imio.smartweb.vocabulary.RemoteNewsFolders", 2)
        vocabulary = get_vocabulary("imio.smartweb.vocabulary.RemoteNewsFolders")
        self.assertEqual(
            vocabulary.getTerm("64f4cbee9a394a018a951f6d94452914").title,
            "News folder : Administration de Belleville",
        )

    @requests_mock.Mocker()
    def test_remote_contacts_error_values(self, m):
        m.get(self.contact_search_url, exc=requests.exceptions.ConnectTimeout)
        self.assertVocabularyLen("imio.smartweb.vocabulary.RemoteContacts", 0)
        m.get(self.contact_search_url, status_code=404)
        self.assertVocabularyLen("imio.smartweb.vocabulary.RemoteContacts", 0)

    @requests_mock.Mocker()
    def test_empty_remote_contacts(self, m):
        m.get(
            self.contact_search_url, text=json.dumps(self.json_empty_contacts_raw_mock)
        )
        self.assertVocabularyLen("imio.smartweb.vocabulary.RemoteContacts", 0)

    def test_alignment(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.Alignment", 4)

    def test_image_size(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.ImageSize", 3)

    def test_concat_category_topics_vocabulary(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.CategoryAndTopics", 126)

    def test_gallery_mode(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.GalleryMode", 2)

    def test_category_and_topics_vocabulary(self):
        self.assertVocabularyLen(
            "imio.smartweb.vocabulary.FilteredCategoryAndTopics", 0
        )

        record = {
            "emploi-mobility": "Emploie dans la mobilité",
            "presse-social": "Presse Sociale",
            "reglement-culture": "Reglement Culture",
        }

        api.portal.set_registry_record(
            name="smartweb.category_and_topics_vocabulary", value=record
        )
        self.assertVocabularyLen(
            "imio.smartweb.vocabulary.FilteredCategoryAndTopics", 3
        )

    @requests_mock.Mocker()
    def test_remote_contact_categories(self, m):
        json_contact_category_raw_mock = get_json(
            "resources/json_contact_category_raw_mock.json"
        )
        with patch(
            "imio.smartweb.core.vocabularies.get_wca_token", return_value="kamoulox"
        ):
            url = f"{config.DIRECTORY_URL}/@vocabularies/collective.taxonomy.contact_category"
            m.get(url, text=json.dumps(json_contact_category_raw_mock))
            self.assertVocabularyLen("imio.smartweb.vocabulary.DirectoryCategories", 25)
            vocabulary = get_vocabulary("imio.smartweb.vocabulary.DirectoryCategories")
            self.assertEqual(
                vocabulary.getTermByToken("cho96vl9ox").title,
                "Commerces et entreprises",
            )

    @requests_mock.Mocker()
    def test_remote_events_types(self, m):
        json_events_types_raw_mock = get_json(
            "resources/json_events_types_raw_mock.json"
        )
        with patch(
            "imio.smartweb.core.vocabularies.get_wca_token", return_value="kamoulox"
        ):
            url = f"{config.EVENTS_URL}/@vocabularies/imio.events.vocabulary.EventTypes"
            m.get(url, text=json.dumps(json_events_types_raw_mock))
            self.assertVocabularyLen("imio.smartweb.vocabulary.EventsTypes", 2)
            vocabulary = get_vocabulary("imio.smartweb.vocabulary.EventsTypes")
            self.assertEqual(
                vocabulary.getTermByToken("activity").title,
                "Activité (extrascolaire, sport, atelier et cours, etc.)",
            )

    @requests_mock.Mocker()
    @freeze_time("2021-11-15")
    def test_events_from_entity(self, m):
        json_entities_raw_mock = get_json(
            "resources/json_events_entities_raw_mock.json"
        )
        url = f"{config.EVENTS_URL}/@search?UID=7c69f9a738ec497c819725c55888ee31"
        m.get(url, text=json.dumps(json_entities_raw_mock))
        json_agendas_raw_mock = get_json("resources/json_events_agendas_raw_mock.json")
        url = f"{config.EVENTS_URL}/imio-events-entity/@search?portal_type=imio.events.Agenda&sort_on=sortable_title&b_size=1000000&metadata_fields=UID"
        m.get(url, text=json.dumps(json_agendas_raw_mock))
        url = f"{config.EVENTS_URL}/@search?selected_agendas=64f4cbee9a394a018a951f6d94452914&selected_agendas=96d3e3299dc74386943e12c4f4fd0b8a&portal_type=imio.events.Event&metadata_fields=category&metadata_fields=topics&metadata_fields=start&metadata_fields=end&metadata_fields=has_leadimage&metadata_fields=breadcrumb&metadata_fields=UID&event_dates.query=2021-11-15&event_dates.range=min&b_size=1000000"
        json_rest_events = get_json("resources/json_rest_events_with_breadcrumbs.json")
        m.get(url, text=json.dumps(json_rest_events))
        vocabulary = get_vocabulary("imio.smartweb.vocabulary.EventsFromEntity")
        self.assertEqual(
            vocabulary.getTermByToken("97902f2e26774a369035117d56381a2a").title,
            "Belleville » Accueil temps libre » Journée de l'ATL",
        )

    @requests_mock.Mocker()
    def test_news_from_entity(self, m):
        json_entities_raw_mock = get_json("resources/json_news_entities_raw_mock.json")
        url = f"{config.NEWS_URL}/@search?UID=7c69f9a738ec497c819725c55888ee32"
        m.get(url, text=json.dumps(json_entities_raw_mock))
        json_newsfolders_raw_mock = get_json(
            "resources/json_news_newsfolder_raw_mock.json"
        )
        url = f"{config.NEWS_URL}/imio-news-entity/@search?portal_type=imio.news.NewsFolder&sort_on=sortable_title&b_size=1000000&metadata_fields=UID"
        m.get(url, text=json.dumps(json_newsfolders_raw_mock))
        url = f"{config.NEWS_URL}/@search?selected_news_folders=64f4cbee9a394a018a951f6d94452914&selected_news_folders=96d3e3299dc74386943e12c4f4fd0b8a&portal_type=imio.news.NewsItem&metadata_fields=category&metadata_fields=topics&metadata_fields=has_leadimage&metadata_fields=breadcrumb&metadata_fields=UID&b_size=1000000"
        json_rest_news = get_json("resources/json_rest_news.json")
        m.get(url, text=json.dumps(json_rest_news))
        vocabulary = get_vocabulary("imio.smartweb.vocabulary.NewsItemsFromEntity")
        self.assertEqual(
            vocabulary.getTermByToken("bfe2b4391a0f4a8db6d8b7fed63d1c4a").title,
            "belleville >> commune >> Restauration de la Bibliothèque",
        )

    def test_sendinblue_button_position(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.SendInBlueButtonPosition", 2)



# -*- coding: utf-8 -*-

from embeddify import Embedder
from embeddify import Plugin
from imio.smartweb.core.contents.sections.views import SectionView
from imio.smartweb.core.interfaces import IArcgisViewUtils
from imio.smartweb.locales import SmartwebMessageFactory as _
from json.decoder import JSONDecodeError
from plone import api
from plone.app.contenttypes.behaviors.leadimage import ILeadImage
from Products.Five.browser import BrowserView
from zope.i18n import translate
from zope.interface import implementer

import json


class ExternalContentView(SectionView):
    def get_embed_external_content(self, width="100%", height=600):
        current_lang = api.portal.get_current_language()[:2]
        plugins = [
            EaglebePlugin(),
            EllohaPlugin(),
            CognitoformPlugin(),
            ArcgisPlugin(),
            UnknowServicePlugin(),
        ]
        extra_params = self.context.external_content_params
        plugin_config = {
            "eaglebeplugin": {"width": width},
            "ellohaplugin": {
                "width": width,
                "current_lang": current_lang,
                "extra_params": extra_params,
            },
            "cognitoformplugin": {
                "width": width,
                "current_lang": current_lang,
                "extra_params": extra_params,
            },
            "arcgisplugin": {
                "width": width,
                "url": self.context.absolute_url(),
                "current_lang": current_lang,
                "extra_params": extra_params,
            },
            "unknowserviceplugin": {"width": width, "current_lang": current_lang},
        }
        embedder = Embedder(plugins=plugins, plugin_config=plugin_config)
        url = self.context.external_content_url
        return embedder(url, config={"width": width})

    def has_leadimage(self):
        if ILeadImage.providedBy(self.context) and getattr(
            self.context, "image", False
        ):
            return True
        return False

    def image(self):
        return f"{self.context.absolute_url()}/@@download/image"


class EaglebePlugin(Plugin):
    def __call__(self, parts, config={}):
        if "app.eaglebe.com" in parts.netloc:
            return f'<iframe class="eaglebe" src="{parts.geturl()}" scrolling="no" width="{config["width"]}">'
        #
        return None


class EllohaPlugin(Plugin):
    def __call__(self, parts, config={}):
        if "reservation.elloha.com" in parts.netloc:
            # sample :
            # {
            #     "ConstellationWidgetContainer": "917c4c52-c997-4077-9135-873a0b2e0c85",
            #     "Idoi": "195ea849-1732-4a69-a051-e7911001cd09"
            # }
            current_lang = config["current_lang"]
            extra_params = config["extra_params"]

            error_message = _(
                "With an elloha plugin, extra params must contain a dictionary with two keys : ConstellationWidgetContainer, Idoi"
            )
            error_message = translate(error_message, target_language=current_lang)
            error_message = f'<div class="elloha elloha_error">{error_message}</div>'

            if (
                extra_params is None
                or extra_params[0] != "{"
                or extra_params[-1] != "}"
            ):
                return error_message
            try:
                res = json.loads(extra_params.lower())
            except JSONDecodeError:
                return error_message

            if (
                res.get("constellationwidgetcontainer") is None
                or res.get("idoi") is None
            ):
                return error_message

            cw1 = res.get("constellationwidgetcontainer")
            cw2 = cw1.replace("-", "")
            idoi = res.get("idoi")
            return (
                f'<div class="elloha">'
                f'<div id="ConstellationWidgetContainer{cw1}" '
                f'style="width:100%" data-id-projet="{cw2}">...</div></div>'
                f'<script type="text/javascript" src="{parts.geturl()}/Scripts/widget-loader.min.js?v=42"></script>'
                f'<script type="text/javascript">var constellationWidgetUrl{cw2}, '
                f"constellationTypeModule{cw2};"
                f'constellationWidgetUrl{cw2} = "https://reservation.elloha.com/Widget/BookingEngine/{cw1}?idoi={idoi}&culture=fr-FR";'
                f'constellationTypeModule{cw2}=1; constellationWidgetLoad("ConstellationWidgetContainer{cw1}");'
                f'constellationWidgetAddEvent(window, "resize", function () {{constellationWidgetSetAppearance("ConstellationWidgetContainer{cw1}");}});</script>'
            )
        #
        return None


class CognitoformPlugin(Plugin):
    def __call__(self, parts, config={}):
        if "cognitoforms" in parts.netloc:
            current_lang = config["current_lang"]
            extra_params = config["extra_params"]

            error_message = _(
                "With a cognitoform plugin, extra params can be void but if you complete it you must specify : scrolling:(yes/no) and overflow:(hidden/scroll/auto)"
            )
            error_message = translate(error_message, target_language=current_lang)
            error_message = (
                f'<div class="cognitoform cognitoform_error">{error_message}</div>'
            )

            if extra_params is None:
                return f'<iframe src="{parts.geturl()}" style="border: 0px none; width: {config["width"]}%; overflow: auto;" scrolling="yes"></iframe>'
            else:
                if extra_params[0] != "{" or extra_params[-1] != "}":
                    return error_message
                try:
                    res = json.loads(extra_params.lower())
                except JSONDecodeError:
                    return error_message

                if res.get("overflow") is None or res.get("scrolling") is None:
                    return error_message

                overflow = res.get("overflow")
                scrolling = res.get("scrolling")
                return f'<iframe src="{parts.geturl()}" style="border: 0px none; width: {config["width"]}%; overflow: {overflow};" scrolling="{scrolling}"></iframe>'
        #
        return None


class ArcgisPlugin(Plugin):
    def __call__(self, parts, config={}):
        if "arcgis" in parts.netloc:
            # url : https://developers.arcgis.com/
            # extra_params : {"portal_item_id":"27a432b0835149e6acd3ac39d0e4349c"}
            current_lang = config["current_lang"]
            extra_params = config["extra_params"]
            url = config["url"]

            error_message = _(
                "With arcgis plugin, extra params must contain a dictionary with one key : portal_item_id"
            )
            error_message = translate(error_message, target_language=current_lang)
            error_message = f'<div class="arcgis arcgis_error">{error_message}</div>'

            if (
                extra_params is None
                or extra_params[0] != "{"
                or extra_params[-1] != "}"
            ):
                return error_message
            try:
                res = json.loads(extra_params.lower())
            except JSONDecodeError:
                return error_message
            if res.get("portal_item_id") is None:
                return error_message

            portal_item_id = res.get("portal_item_id")
            msg = _("Consult the map")
            msg = translate(msg, target_language=current_lang)
            return (
                f'<a href="{url}/view_arcgis?portal_item_id={portal_item_id}">{msg}</a>'
            )
        #
        return None


class UnknowServicePlugin(Plugin):
    def __call__(self, parts, config={}):
        current_lang = config["current_lang"]
        msg = _("Unknow service")
        msg = translate(msg, target_language=current_lang)
        return f'<p class="unknow_service">{msg}</p>'


@implementer(IArcgisViewUtils)
class ArcgisView(BrowserView):
    _footer = None

    @property
    def get_portal_item_id(self):
        return self.request.portal_item_id



# -*- coding: utf-8 -*-

from AccessControl.unauthorized import Unauthorized
from bs4 import BeautifulSoup
from collective.geolocationbehavior.geolocation import IGeolocatable
from functools import reduce
from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import clear_cache
from imio.smartweb.core.tests.utils import make_named_image
from imio.smartweb.core.tests.utils import get_sections_types
from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.namedfile.file import NamedBlobFile
from plone.namedfile.file import NamedBlobImage
from plone.protect.authenticator import createToken
from time import sleep
from z3c.relationfield import RelationValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import modified


class TestSections(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        # Number of sections where there is a title if section is empty.
        # sectionHTML,...
        self.NUMBER_OF_EMPTY_SECTIONS = len(get_sections_types("empty_section"))

        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            id="page",
        )

    def test_redirection(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionText",
            title="Section text",
            id="text",
        )
        getMultiAdapter((section, self.request), name="view")()
        self.assertEqual(
            self.request.response.headers["location"],
            "http://nohost/plone/page#section-text",
        )

    def test_modification_date(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionText",
            title="Section text",
        )
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionGallery",
            title="Section Gallery",
        )
        first_modification = section.ModificationDate()
        sleep(1)
        # adding an image to a section causes a reindex of the section, thus
        # changes its last modification date
        api.content.create(
            container=section,
            type="Image",
            title="Image",
        )
        next_modification = section.ModificationDate()
        self.assertNotEqual(first_modification, next_modification)

    def test_files_informations(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionFiles",
            title="Section files",
        )
        file_obj = api.content.create(
            container=section,
            type="File",
            title="My file",
        )
        file_obj.file = NamedBlobFile(data="file data", filename="file.txt")
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn("1 KB", view())

    def test_video_section(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionVideo",
            title="Section video",
        )
        section.video_url = "https://www.youtube.com/watch?v=_dOAthafoGQ"
        view = queryMultiAdapter((section, self.request), name="view")
        embedded_video = view.get_embed_video()
        self.assertIn("iframe", embedded_video)
        self.assertIn(
            "https://www.youtube.com/embed/_dOAthafoGQ?feature=oembed", embedded_video
        )

    def test_external_content_section(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        with self.assertRaises(Unauthorized):
            api.content.create(
                container=self.page,
                type="imio.smartweb.SectionExternalContent",
                title="Section External Content",
            )
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionExternalContent",
            title="Section External Content",
        )
        section.external_content_url = (
            "https://app.eaglebe.com/fr-be/map/la%20louvi%C3%A8re"
        )
        view = queryMultiAdapter((section, self.request), name="view")
        embedded_content = view.get_embed_external_content()
        self.assertIn("iframe", embedded_content)
        self.assertIn('class="eaglebe"', embedded_content)
        self.assertIn('scrolling="no"', embedded_content)
        self.assertIn(
            "https://app.eaglebe.com/fr-be/map/la%20louvi%C3%A8re", embedded_content
        )

        section.external_content_url = "http://www.perdu.com"
        view = queryMultiAdapter((section, self.request), name="view")
        embedded_content = view.get_embed_external_content()
        self.assertNotIn("iframe", embedded_content)
        self.assertNotIn("class='eaglebe'", embedded_content)
        self.assertIn('<p class="unknow_service">Unknow service</p>', embedded_content)

    def test_map_section(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionMap",
            title="Section map",
        )
        IGeolocatable(section).geolocation = Geolocation(latitude="4.5", longitude="45")
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn("Section map", view())
        self.assertIn('class="pat-leaflet map"', view())

    def test_selections_section(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionSelections",
            title="Section selections",
        )
        page2 = api.content.create(
            container=self.portal, type="imio.smartweb.Page", title="Page 2"
        )
        intids = getUtility(IIntIds)
        section.selected_items = [
            RelationValue(intids.getId(page2)),
        ]
        view = queryMultiAdapter((section, self.request), name="table_view")
        self.assertIn("Page 2", [item[0]["title"] for item in view.items()])
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn("Page 2", view())

    def test_collection_section(self):
        portal_page = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            title="Portal page",
        )
        section_gallery = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionGallery",
            title="Section gallery",
        )
        for i in range(0, 10):
            api.content.create(
                container=section_gallery,
                type="Image",
                title=f"Image{i}",
            )
        query = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.selection.any",
                "v": ["Image"],
            }
        ]
        collection = api.content.create(
            container=self.portal,
            type="Collection",
            title="My collection",
        )
        collection.setQuery(query)
        self.assertEqual(len(collection.results()), 10)
        section_collection = api.content.create(
            container=portal_page,
            type="imio.smartweb.SectionCollection",
            title="Section collection",
        )
        intids = getUtility(IIntIds)
        section_collection.collection = RelationValue(intids.getId(collection))
        section_collection.max_nb_batches = 2
        section_collection.nb_results_by_batch = 3
        view = queryMultiAdapter((section_collection, self.request), name="table_view")
        items = view.items()
        self.assertEqual(len(items), 2)
        self.assertEqual(len(items[0]), 3)
        self.assertEqual(reduce(lambda count, r: count + len(r), items, 0), 6)
        section_collection.nb_results_by_batch = 1
        section_collection.max_nb_batches = 4
        view = queryMultiAdapter((section_collection, self.request), name="table_view")
        self.assertEqual(len(view.items()), 4)

    def test_sections_ordering(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        alsoProvides(self.request, IImioSmartwebCoreLayer)
        api.content.create(
            container=page,
            type="imio.smartweb.SectionGallery",
            title="Section 1",
            id="section1",
        )
        api.content.create(
            container=page,
            type="imio.smartweb.SectionText",
            title="Section 2",
            id="section2",
        )
        self.assertListEqual(page.objectIds(), ["section1", "section2"])
        self.request.form["_authenticator"] = createToken()
        self.request.form["id"] = "section2"
        self.request.form["delta"] = "-1"
        self.request.form["orderedSectionsIds"] = '["section1", "section2"]'
        self.request.environ["REQUEST_METHOD"] = "POST"
        getMultiAdapter((page, self.request), name="reorder-section")()
        self.assertListEqual(page.objectIds(), ["section2", "section1"])

        self.request.form["id"] = "section1"
        self.request.form["delta"] = "-1"
        # wrong ordered sections ids
        self.request.form["orderedSectionsIds"] = '["section1", "section2"]'
        getMultiAdapter((page, self.request), name="reorder-section")()
        # nothing changes
        self.assertListEqual(page.objectIds(), ["section2", "section1"])

    def test_empty_sections(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        api.content.transition(page, "publish")
        section_types = get_sections_types()
        for section_type in section_types:
            api.content.create(
                container=page,
                type=section_type,
                title="Title of my {}".format(section_type),
            )
        video_section = getattr(page, "title-of-my-imio-smartweb-sectionvideo")
        video_section.video_url = "https://www.youtube.com/watch?v=_dOAthafoGQ"
        selections_section = getattr(
            page, "title-of-my-imio-smartweb-sectionselections"
        )
        intids = getUtility(IIntIds)
        selections_section.selected_items = [
            RelationValue(intids.getId(self.page)),
        ]
        view = queryMultiAdapter((page, self.request), name="full_view")()
        self.assertEqual(
            view.count('<h2 class="section-title">Title of my '), len(section_types)
        )
        logout()
        view = queryMultiAdapter((page, self.request), name="full_view")()
        self.assertEqual(view.count('<h2 class="section-title">Title of my '), 5)
        login(self.portal, "test")
        gallery_section = getattr(page, "title-of-my-imio-smartweb-sectiongallery")
        api.content.create(
            container=gallery_section,
            type="Image",
            title="Image",
        )
        files_section = getattr(page, "title-of-my-imio-smartweb-sectionfiles")
        file_obj = api.content.create(
            container=files_section,
            type="File",
            title="My file",
        )
        file_obj.file = NamedBlobFile(data="file data", filename="file.txt")
        links_section = getattr(page, "title-of-my-imio-smartweb-sectionlinks")
        api.content.create(
            container=links_section,
            type="imio.smartweb.BlockLink",
            title="My link",
        )
        view = queryMultiAdapter((page, self.request), name="full_view")()
        self.assertEqual(
            view.count('<h2 class="section-title">Title of my '), len(section_types)
        )

    def test_sections_titles(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        api.content.transition(page, "publish")
        section_types = get_sections_types()
        for section_type in section_types:
            api.content.create(
                container=page,
                type=section_type,
                title="Title of my {}".format(section_type),
            )
        video_section = getattr(page, "title-of-my-imio-smartweb-sectionvideo")
        video_section.video_url = "https://www.youtube.com/watch?v=_dOAthafoGQ"
        gallery_section = getattr(page, "title-of-my-imio-smartweb-sectiongallery")
        api.content.create(
            container=gallery_section,
            type="Image",
            title="Image",
        )
        files_section = getattr(page, "title-of-my-imio-smartweb-sectionfiles")
        file_obj = api.content.create(
            container=files_section,
            type="File",
            title="My file",
        )
        file_obj.file = NamedBlobFile(data="file data", filename="file.txt")
        links_section = getattr(page, "title-of-my-imio-smartweb-sectionlinks")
        api.content.create(
            container=links_section,
            type="imio.smartweb.BlockLink",
            title="My link",
        )
        selections_section = getattr(
            page, "title-of-my-imio-smartweb-sectionselections"
        )
        intids = getUtility(IIntIds)
        selections_section.selected_items = [
            RelationValue(intids.getId(self.page)),
        ]

        view = queryMultiAdapter((page, self.request), name="full_view")()
        self.assertEqual(
            view.count('<h2 class="section-title">Title of my '), len(section_types)
        )
        # test hide_title
        for section_id in page.objectIds():
            section = getattr(page, section_id)
            section.hide_title = True
        # hide_title in login mode (add some specific css class)
        view = queryMultiAdapter((page, self.request), name="full_view")()
        self.assertEqual(
            view.count('<h2 class="hidden-section-title hide-in-preview">Title of my '),
            self.NUMBER_OF_EMPTY_SECTIONS,
        )
        # hide_title in logout mode (no more <h2> / title)
        logout()
        view = queryMultiAdapter((page, self.request), name="full_view")()
        self.assertEqual(view.count("</h2>"), 0)

    def test_sections_titles_display_switch(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        api.content.transition(page, "publish")
        # We can't edit title visibility of Contact & Text sections.
        # And visibility of these sections titles is False.
        for section_type in [
            "imio.smartweb.SectionContact",
            "imio.smartweb.SectionText",
        ]:
            section = api.content.create(
                container=page,
                type=section_type,
                title="Title of my {}".format(section_type),
            )
            section.hide_title = True
            view = queryMultiAdapter((section, self.request), name="hide_section_title")
            view.hide_section_title()
            self.assertTrue(section.hide_title)
            view = queryMultiAdapter((section, self.request), name="show_section_title")
            view.show_section_title()
            self.assertTrue(section.hide_title)
            view = queryMultiAdapter((page, self.request), name="full_view")
            self.assertNotIn("Show title", view())

        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionGallery",
            title="Title of my galery",
        )
        view = queryMultiAdapter((section, self.request), name="hide_section_title")
        view.hide_section_title()
        self.assertTrue(section.hide_title)
        view = queryMultiAdapter((section, self.request), name="show_section_title")
        view.show_section_title()
        self.assertFalse(section.hide_title)

    def test_sections_collapsible(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        api.content.transition(page, "publish")
        section_types = get_sections_types()
        for section_type in section_types:
            api.content.create(
                container=page,
                type=section_type,
                title="Title of my {}".format(section_type),
            )
        video_section = getattr(page, "title-of-my-imio-smartweb-sectionvideo")
        video_section.video_url = "https://www.youtube.com/watch?v=_dOAthafoGQ"
        gallery_section = getattr(page, "title-of-my-imio-smartweb-sectiongallery")
        api.content.create(
            container=gallery_section,
            type="Image",
            title="Image",
        )
        files_section = getattr(page, "title-of-my-imio-smartweb-sectionfiles")
        file_obj = api.content.create(
            container=files_section,
            type="File",
            title="My file",
        )
        file_obj.file = NamedBlobFile(data="file data", filename="file.txt")
        links_section = getattr(page, "title-of-my-imio-smartweb-sectionlinks")
        api.content.create(
            container=links_section,
            type="imio.smartweb.BlockLink",
            title="My link",
        )
        postit_section = getattr(page, "title-of-my-imio-smartweb-sectionpostit")
        postits = [{"title": "postit1", "subtitle": "", "description": ""}]
        postit_section.postits = postits

        selections_section = getattr(
            page, "title-of-my-imio-smartweb-sectionselections"
        )
        intids = getUtility(IIntIds)
        selections_section.selected_items = [
            RelationValue(intids.getId(self.page)),
        ]

        for section_id in page.objectIds():
            section = getattr(page, section_id)
            section.collapsible_section = True
            modified(section)
            self.assertFalse(section.hide_title)
        view = queryMultiAdapter((page, self.request), name="full_view")()
        soup = BeautifulSoup(view)
        collapsables = soup.select("div.body-section.collapse")
        self.assertEqual(len(collapsables), len(section_types))

    def test_sections_collapsible_hide_title(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionHTML",
            title="Section HTML",
        )
        view = queryMultiAdapter((page, self.request), name="full_view")()
        self.assertIn("Hide title", view)
        section.collapsible_section = True
        modified(section)
        view = queryMultiAdapter((page, self.request), name="full_view")()
        self.assertNotIn("Hide title", view)
        self.assertFalse(section.hide_title)
        view = queryMultiAdapter((section, self.request), name="hide_section_title")
        view.hide_section_title()
        self.assertFalse(section.hide_title)

    def test_sections_worflows(self):
        wf_tool = api.portal.get_tool("portal_workflow")
        section_types = get_sections_types()
        for section_type in section_types:
            obj = api.content.create(
                container=self.page,
                type=section_type,
                title="Title of my {}".format(section_type),
            )
            chain = wf_tool.getChainFor(obj)
            self.assertEqual(chain, ())

    def test_section_html(self):
        section_html = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionHTML",
            title="Section html",
            id="section-html",
            html="",
        )
        section_html.html = (
            '<div><h1>Perdu.com</h1><embed src="http://www.perdu.com"></embed></div>'
        )
        modified(section_html)
        page_html = getMultiAdapter((self.page, self.request), name="full_view")()
        self.assertIn("Perdu.com", page_html)
        self.assertNotIn("iframe", page_html)

    def test_link_section(self):
        links_section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionLinks",
            title="Section links",
        )
        api.content.transition(self.page, "publish")
        link = api.content.create(
            container=links_section,
            type="imio.smartweb.BlockLink",
            title="My link",
        )
        view = getMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn(
            '<a class="table_image no-image" href="http://nohost/plone/page/section-links/my-link" target="">',
            view(),
        )
        link.image = NamedBlobImage(**make_named_image())
        self.assertIn(
            '<a class="table_image" href="http://nohost/plone/page/section-links/my-link" target="">',
            view(),
        )
        link.open_in_new_tab = True
        self.assertNotIn(
            '<a class="table_image" href="http://nohost/plone/page/section-links/my-link" target="_blank">',
            view(),
        )
        logout()
        clear_cache(self.request)
        view = getMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn(
            '<a class="table_image" href="http://nohost/plone/page/section-links/my-link" target="_blank">',
            view(),
        )

    def test_background_style(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionText",
            title="Section text",
        )
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        self.assertEqual(view.background_style(section), "")
        section.background_image = NamedBlobImage(**make_named_image())
        self.assertIn(
            "background-image:url(http://nohost/plone/page/section-text/@@images/background_image-768",
            view.background_style(section),
        )

    def test_get_class(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionText",
            title="Section text",
        )
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        self.assertEqual(view.get_class(section), "sectiontext")
        section.css_class = "my-css"
        self.assertEqual(view.get_class(section), "sectiontext my-css")
        section.bootstrap_css_class = "col-sm-3"
        self.assertEqual(view.get_class(section), "sectiontext my-css col-sm-3")
        section.background_image = NamedBlobImage(**make_named_image())
        self.assertEqual(
            view.get_class(section), "sectiontext my-css col-sm-3 with-background"
        )



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform import directives
from zope import schema
from zope.interface import implementer


class ISectionText(ISection):
    """Marker interface and Dexterity Python Schema for SectionText"""

    directives.order_before(alignment="IVersionable.changeNote")
    alignment = schema.Choice(
        title=_("Image alignment"),
        description=_("Select where you want to display the image"),
        source="imio.smartweb.vocabulary.Alignment",
        default="left",
        required=True,
    )
    directives.order_after(image_size="alignment")
    image_size = schema.Choice(
        title=_("Image size"),
        description=_("Select image size"),
        source="imio.smartweb.vocabulary.ImageSize",
        default="preview",
        required=False,
    )


@implementer(ISectionText)
class SectionText(Section):
    """SectionText class"""

    can_toggle_title_visibility = False



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets.httpheaders import HeaderViewlet


class ArcgisHeaderViewlet(HeaderViewlet):
    def update(self):
        super(ArcgisHeaderViewlet, self).update()

    # def getHeaders(self):
    #     import pdb;pdb.set_trace()
    #     result = super(ArcgisHeaderViewlet, self).getHeaders()
    #     result.append(("prefix", "og: http://ogp.me/ns#"))
    #     return result



# -*- coding: utf-8 -*-

from Acquisition import aq_inner
from imio.smartweb.core.contents import IFolder
from imio.smartweb.core.interfaces import IViewWithoutLeadImage
from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.content.browser.contents import ContextInfo
from plone.app.content.utils import json_dumps
from plone.app.content.utils import json_loads
from plone.app.contenttypes.browser.folder import FolderView as BaseFolderView
from plone.app.contenttypes.interfaces import ICollection
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form.browser.radio import RadioFieldWidget
from z3c.form.contentprovider import ContentProviders
from z3c.form.field import Fields
from z3c.form.form import EditForm
from z3c.form.interfaces import IFieldsAndContentProvidersForm
from zope.component import queryMultiAdapter
from zope.contentprovider.provider import ContentProviderBase
from zope.interface import implementer


class FolderContextInfo(ContextInfo):
    """View used to fetch informations for folder_contents"""

    def __call__(self):
        json = super(FolderContextInfo, self).__call__()
        brain = self.context.get_default_item()
        if brain:
            infos = json_loads(json)
            infos["defaultPage"] = brain.getId
            json = json_dumps(infos)
        return json


@implementer(IViewWithoutLeadImage)
class FolderView(BaseFolderView):
    """ """

    show_images = False

    def results(self, **kwargs):
        """
        Gets results for folder listings with exclude_from_parent_listing into
        filters and without taking care of friendly_types
        """
        # Extra filter
        kwargs.update(self.request.get("contentFilter", {}))
        kwargs.setdefault("batch", True)
        kwargs.setdefault("b_size", self.b_size)
        kwargs.setdefault("b_start", self.b_start)
        kwargs.setdefault("orphan", 1)
        kwargs.setdefault("exclude_from_parent_listing", False)

        listing = aq_inner(self.context).restrictedTraverse("@@folderListing", None)
        return listing(**kwargs)

    def blocks_results(self, **kwargs):
        """
        Gets results for blocks folder view, combining standard folder listing
        results and quick access contents (without duplicates)
        """
        results = self.results(batch=False)
        if self.context.quick_access_items is None:
            return {"results": results, "quick_access": []}
        quick_access_uids = [
            item.to_object.UID()
            for item in self.context.quick_access_items
            if item.isBroken() is False
        ]
        quick_access_brains = api.content.find(
            UID=quick_access_uids,
            sort_on="sortable_title",
        )
        paths = [item.getPath() for item in results]
        # Use path instead of uuid in comparison because uuid can wake up object.
        quick_access_brains = [
            brain for brain in quick_access_brains if brain.getPath() not in paths
        ]
        return {"results": results, "quick_access": quick_access_brains}

    def get_scale_url(self, item, scale):
        request = self.request
        return get_scale_url(item, request, "image", scale)


@implementer(IViewWithoutLeadImage)
class FolderViewWithImages(FolderView):
    """ """

    show_images = True


class DefaultElementTextView(ContentProviderBase):
    render = ViewPageTemplateFile("element_view.pt")


@implementer(IFieldsAndContentProvidersForm, IViewWithoutLeadImage)
class ElementView(EditForm):
    """ """

    label = _("Form to choose item to be displayed as the home page of the folder")
    contentProviders = ContentProviders()
    contentProviders["defaultElementText"] = DefaultElementTextView
    contentProviders["defaultElementText"].position = 0

    def __call__(self):
        if api.user.is_anonymous():
            default_item = self.context.get_default_item(object=True)
            if default_item:
                # we choose the view depending on content type (pages or collection)
                view_name = (
                    ICollection.providedBy(default_item)
                    and "facetednavigation_view"
                    or "full_view"
                )
                return queryMultiAdapter((default_item, self.request), name=view_name)()
            # Element view with no default item selected -> fallback to summary_view
            return queryMultiAdapter(
                (self.context, self.request), name="summary_view"
            )()
        return super(ElementView, self).__call__()

    @property
    def fields(self):
        fields = Fields(IFolder).select("default_page_uid")
        fields["default_page_uid"].required = True
        fields["default_page_uid"].widgetFactory = RadioFieldWidget
        return fields

    @button.buttonAndHandler(_("Apply"), name="apply")
    def handleApply(self, action):
        data, errors = self.extractData()
        old_default_page = self.context.get_default_item(object=True)
        changes = self.applyChanges(data)
        if changes:
            self.context.set_default_item(old_default_item=old_default_page)
            self.status = self.successMessage
        else:
            self.status = self.noChangesMessage


@implementer(IViewWithoutLeadImage)
class SummaryView(FolderView):
    """ """

    show_images = False


@implementer(IViewWithoutLeadImage)
class SummaryViewWithImages(SummaryView):
    """ """

    show_images = True



# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup
from imio.smartweb.core.tests.utils import make_named_image
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.namedfile.file import NamedBlobImage
from plone.testing.zope import Browser
from zope.component import getMultiAdapter

import transaction


class TestText(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            id="page",
        )

    def test_toggle_title_visibility(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        api.content.transition(page, "publish")
        # We can't edit title visibility of a "Text" section.
        # And visibility of text title is False.
        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionText",
            title="Title of my text",
        )
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/edit".format(section.absolute_url()))
        content = browser.contents
        soup = BeautifulSoup(content)
        hide_title_true = soup.find(id="form-widgets-hide_title-0")
        self.assertIsNotNone(hide_title_true)
        self.assertEqual(len(hide_title_true), 0)
        self.assertEqual(hide_title_true["type"], "hidden")
        self.assertEqual(hide_title_true["value"], "selected")
        hide_title_false = soup.find(id="form-widgets-hide_title-1")
        self.assertIsNone(hide_title_false)

        browser.open("{}/++add++{}".format(page.absolute_url(), section.portal_type))
        content = browser.contents
        soup = BeautifulSoup(content)
        hide_title_true = soup.find(id="form-widgets-hide_title-0")
        self.assertIsNotNone(hide_title_true)
        self.assertEqual(len(hide_title_true), 0)
        self.assertEqual(hide_title_true["type"], "hidden")
        self.assertEqual(hide_title_true["value"], "selected")
        hide_title_false = soup.find(id="form-widgets-hide_title-1")
        self.assertIsNone(hide_title_false)

    def test_lead_image(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionText",
            title="Title of my text",
        )
        section.alignment = "left"
        section.image_size = "large"
        view = getMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn(
            '<div class="body-section figure-left figure-large  no-image"', view()
        )
        self.assertNotIn("<figure", view())
        self.assertNotIn("@@images/image/large", view())
        self.assertNotIn("figcaption", view())

        section.image = NamedBlobImage(**make_named_image())
        view = getMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn('<div class="body-section figure-left figure-large "', view())
        self.assertIn("<figure", view())
        self.assertIn("@@images/image-768-", view())
        self.assertNotIn("figcaption", view())

        section.image_caption = "Kamoulox"
        view = getMultiAdapter((self.page, self.request), name="full_view")
        # Assert section text has lead image
        self.assertIn('<div class="body-section figure-left figure-large "', view())
        self.assertIn("<figure", view())
        self.assertIn("figcaption", view())

        section.alignment = "right"
        view = getMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn('<div class="body-section figure-right figure-large "', view())

        section.image_size = "mini"
        view = getMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn('<div class="body-section figure-right figure-mini "', view())
        self.assertIn("@@images/image-200-", view())



# -*- coding: utf-8 -*-

from Acquisition import aq_parent
from collective.taxonomy.interfaces import ITaxonomy
from imio.smartweb.core.config import WCA_URL
from imio.smartweb.core.contents import IFolder
from more_itertools import chunked
from plone import api
from plone.app.multilingual.interfaces import ILanguageRootFolder
from plone.dexterity.interfaces import IDexterityContent
from Products.CMFPlone.defaultpage import get_default_page
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone.utils import base_hasattr
from zope.component import getSiteManager
from zope.component import queryMultiAdapter

import hashlib
import json
import os
import requests


def get_category(context):
    if not base_hasattr(context, "category_name"):
        return
    field_name = "taxonomy_{}".format(context.category_name)
    taxonomy_name = "collective.taxonomy.{}".format(context.category_name)
    term = getattr(context, field_name)
    if not term:
        return
    current_lang = api.portal.get_current_language()[:2]
    sm = getSiteManager()
    utility = sm.queryUtility(ITaxonomy, name=taxonomy_name)
    value = utility.translate(
        term,
        context=context,
        target_language=current_lang,
    )
    return value


def get_categories():
    sm = getSiteManager()
    return sm.queryUtility(ITaxonomy, name="collective.taxonomy.page_category")


def concat_voca_term(term1, term2):
    return "{0}-{1}".format(term1, term2)


def concat_voca_title(title1, title2):
    return "{0} - {1}".format(title1, title2)


def get_json(url, auth=None):
    headers = {"Accept": "application/json"}
    if auth is not None:
        headers["Authorization"] = auth
    try:
        response = requests.get(url, headers=headers, timeout=5)
    except Exception:
        return None
    if response.status_code != 200:
        return None
    return json.loads(response.text)


def get_wca_token(client_id, client_secret):
    username = os.environ.get("RESTAPI_USER_USERNAME")
    password = os.environ.get("RESTAPI_USER_PASSWORD")

    payload = {
        "grant_type": "password",
        "client_id": client_id,
        "client_secret": client_secret,
        "username": username,
        "password": password,
        "scope": ["openid"],
    }
    if not client_id or not client_secret:
        return (username, password)
    headers = {
        "Content-Type": "application/x-www-form-urlencoded",
    }
    response = requests.post(WCA_URL, headers=headers, data=payload)
    id_token = response.json().get("id_token")
    return "Bearer {0}".format(id_token)


def hash_md5(text):
    return hashlib.md5(text.encode()).hexdigest()


def safe_html(html):
    if not html:
        return
    transforms = api.portal.get_tool("portal_transforms")
    data = transforms.convertTo(target_mimetype="text/x-html-safe", orig=html)
    output = data.getData()
    return output


def batch_results(iterable, batch_size):
    return list(chunked(iterable, batch_size, strict=False))


def reindexParent(obj, event):
    parent = aq_parent(obj)
    if parent is not None:
        # in some cases (ex: relation breaking), we do not get the object in
        # its acquisition chain
        parent.reindexObject()


def get_default_content_id(obj):
    if IPloneSiteRoot.providedBy(obj) or ILanguageRootFolder.providedBy(obj):
        # Plone / LRF default page
        item_id = get_default_page(obj)
        return item_id and item_id or ""
    elif IFolder.providedBy(obj):
        # Our folder default page
        item = obj.get_default_item()
        return item and item.getId or ""


def get_scale_url(context, request, fieldname, scale):
    if IDexterityContent.providedBy(context):
        # get scale url on an object
        images_view = queryMultiAdapter((context, request), name="images")
        if images_view is None:
            return ""
        scale = images_view.scale(fieldname, scale)
        if scale is None:
            return ""
        return scale.url
    else:
        # get scale url on a brain
        # In this case, we need a modification hash to handle croppings, because
        # catalog does not handle them correctly.
        # See https://github.com/collective/plone.app.imagecropping/issues/129
        brain = context
        if fieldname == "image" and not brain.has_leadimage:
            return ""
        modification_date = brain.ModificationDate
        if callable(modification_date):
            # brain in content listing for example
            modification_date = modification_date()
        modified_hash = hash_md5(modification_date)
        url = f"{brain.getURL()}/@@images/{fieldname}/{scale}?cache_key={modified_hash}"
        return url



# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup
from datetime import datetime
from datetime import timedelta
from freezegun import freeze_time
from imio.smartweb.core.contents.sections.contact.utils import ContactProperties
from imio.smartweb.core.contents.sections.contact.utils import formatted_schedule
from imio.smartweb.core.contents.sections.contact.utils import get_schedule_for_today
from imio.smartweb.core.contents.sections.views import SECTION_ITEMS_HASH_KEY
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_json
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.zope import Browser
from time import sleep
from zope.annotation.interfaces import IAnnotations
from zope.component import queryMultiAdapter

import json
import requests
import requests_mock
import transaction


class TestSectionContact(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            id="page",
        )
        self.json_contact = get_json("resources/json_contact_raw_mock.json")
        self.json_no_contact = get_json("resources/json_no_contact_raw_mock.json")
        self.json_contact_images = get_json(
            "resources/json_contact_images_raw_mock.json"
        )
        self.json_no_image = get_json("resources/json_contact_no_image_raw_mock.json")

    @requests_mock.Mocker()
    def test_contact(self, m):
        contact = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionContact",
            title="My contact",
        )
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        self.assertIn("My contact", view())
        contact_view = queryMultiAdapter((contact, self.request), name="view")
        self.assertIsNone(contact_view.contacts())
        authentic_contact_uid = "2dc381f0fb584381b8e4a19c84f53b35"
        contact.related_contacts = [authentic_contact_uid]
        contact_search_url = (
            "http://localhost:8080/Plone/@search?UID={}&fullobjects=1".format(
                authentic_contact_uid
            )
        )
        contact_images_url = "http://localhost:8080/Plone/{}/@search?portal_type=Image&path.depth=1&metadata_fields=modified".format(
            authentic_contact_uid
        )
        m.get(contact_search_url, exc=requests.exceptions.ConnectTimeout)
        self.assertIsNone(contact_view.contacts())
        m.get(contact_search_url, status_code=404)
        self.assertIsNone(contact_view.contacts())
        m.get(contact_search_url, text=json.dumps(self.json_no_contact))
        self.assertIsNone(contact_view.contacts())
        m.get(contact_search_url, text=json.dumps(self.json_contact))
        self.assertIsNotNone(contact_view.contacts())
        json_contact = ContactProperties(self.json_contact.get("items")[0])
        self.assertEqual(json_contact.contact_type_class, "contact-type-organization")
        self.assertNotIn("contact_titles", view())
        self.assertIn("contact_address", view())
        self.assertIn("contact_informations", view())
        self.assertIn("schedule", view())
        self.assertNotIn("contact_gallery", view())
        contact.visible_blocks = ["titles", "gallery"]
        self.assertIn("contact_titles", view())
        self.assertNotIn("contact_address", view())
        self.assertNotIn("contact_informations", view())
        self.assertNotIn("schedule", view())
        contact.visible_blocks = ["contact_informations", "address", "schedule"]
        self.assertEqual(view().count("<h2"), 4)
        contact.visible_blocks = [
            "titles",
            "contact_informations",
            "address",
            "map",
            "schedule",
        ]
        self.assertIn('class="pat-leaflet map"', view())
        self.assertEqual(view().count("<h2"), 2)
        self.assertEqual(view().count("<h3"), 3)

        self.assertNotIn("contact_description", view())
        contact.visible_blocks = ["description"]
        self.assertIn("contact_description", view())
        self.assertIn(
            "Description <strong>avec gras</strong> et <br/> retours à la ligne",
            view(),
        )

        contact.visible_blocks = ["titles", "gallery"]
        m.get(contact_images_url, text=json.dumps(self.json_contact_images))
        self.assertIn("contact_gallery", view())

        json_contact = ContactProperties(self.json_contact.get("items")[0])
        images = json_contact.images(contact.image_scale, contact.nb_results_by_batch)
        self.assertEqual(len(images[0]), 2)

        json_contact = ContactProperties(self.json_contact.get("items")[0])
        m.get(contact_images_url, text=json.dumps(self.json_no_image))
        images = json_contact.images(contact.image_scale, contact.nb_results_by_batch)
        self.assertIsNone(images)

        m.get(contact_images_url, status_code=404)
        images = json_contact.images(contact.image_scale, contact.nb_results_by_batch)
        self.assertIsNone(images)

        m.get(contact_images_url, exc=requests.exceptions.ConnectTimeout)
        images = json_contact.images(contact.image_scale, contact.nb_results_by_batch)
        self.assertIsNone(images)

    def test_toggle_title_visibility(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        api.content.transition(page, "publish")
        # We can't edit title visibility of a "Contact" section.
        # And visibility of contact title is False.
        section = api.content.create(
            container=page,
            type="imio.smartweb.SectionContact",
            title="Title of my contact",
        )
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/edit".format(section.absolute_url()))
        content = browser.contents
        soup = BeautifulSoup(content)
        hide_title_true = soup.find(id="form-widgets-hide_title-0")
        self.assertIsNotNone(hide_title_true)
        self.assertEqual(len(hide_title_true), 0)
        self.assertEqual(hide_title_true["type"], "hidden")
        self.assertEqual(hide_title_true["value"], "selected")
        hide_title_false = soup.find(id="form-widgets-hide_title-1")
        self.assertIsNone(hide_title_false)

        browser.open("{}/++add++{}".format(page.absolute_url(), section.portal_type))
        content = browser.contents
        soup = BeautifulSoup(content)
        hide_title_true = soup.find(id="form-widgets-hide_title-0")
        self.assertIsNotNone(hide_title_true)
        self.assertEqual(len(hide_title_true), 0)
        self.assertEqual(hide_title_true["type"], "hidden")
        self.assertEqual(hide_title_true["value"], "selected")
        hide_title_false = soup.find(id="form-widgets-hide_title-1")
        self.assertIsNone(hide_title_false)

    @requests_mock.Mocker()
    def test_opening_informations(self, m):
        contact = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionContact",
            title="My contact",
        )
        contact_view = queryMultiAdapter((contact, self.request), name="view")
        json_contact = contact_view.get_contact_properties(
            self.json_contact.get("items")[0]
        )
        self.assertIsNotNone(json_contact.get_opening_informations())

        today = datetime.now()
        today_str = today.strftime("%Y-%m-%d")
        yesterday_str = (today - timedelta(days=1)).strftime("%Y-%m-%d")
        tomorrow_str = (today + timedelta(days=1)).strftime("%Y-%m-%d")

        self.json_contact["items"][0]["exceptional_closure"] = [
            {"date": today_str, "title": "Exceptional closure !"}
        ]
        json_contact = contact_view.get_contact_properties(
            self.json_contact.get("items")[0]
        )
        self.assertIsNotNone(json_contact.get_opening_informations())

        today = datetime.now().strftime("%Y-%m-%d")
        self.json_contact["items"][0]["multi_schedule"][0]["dates"] = [
            {"end_date": tomorrow_str, "start_date": yesterday_str}
        ]
        json_contact = contact_view.get_contact_properties(
            self.json_contact.get("items")[0]
        )
        self.assertIsNotNone(json_contact.get_opening_informations())

        self.json_contact["items"][0]["multi_schedule"][0]["dates"] = [
            {"end_date": yesterday_str, "start_date": yesterday_str}
        ]
        json_contact = contact_view.get_contact_properties(
            self.json_contact.get("items")[0]
        )
        self.assertIsNotNone(json_contact.get_opening_informations())

    # {'afternoonend': '', 'afternoonstart': '', 'comment': 'vendredi : apéro à midi', 'morningend': '11:00', 'morningstart': '08:30'}
    def test_get_schedule_for_today(self):
        schedule = {
            "morningstart": "08:30",
            "morningend": "12:00",
            "afternoonstart": "13:00",
            "afternoonend": "17:00",
            "comment": "",
        }
        with freeze_time("2021-09-14 8:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertIn("Open at", result)
        with freeze_time("2021-09-14 10:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Open")
        with freeze_time("2021-09-14 12:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Lunch time")
        with freeze_time("2021-09-14 18:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Closed")

        schedule = {
            "morningstart": "08:30",
            "morningend": "12:00",
            "afternoonstart": "",
            "afternoonend": "",
            "comment": "",
        }
        with freeze_time("2021-09-14 7:30:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual("Open at  08:30", result)
        with freeze_time("2021-09-14 8:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertIn("Open at", result)
        with freeze_time("2021-09-14 10:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Open")
        with freeze_time("2021-09-14 12:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Closed")

        schedule = {
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "13:00",
            "afternoonend": "17:00",
            "comment": "",
        }
        with freeze_time("2021-09-14 13:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertIn(result, "Open at")
        with freeze_time("2021-09-14 14:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Open")
        with freeze_time("2021-09-14 17:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Closed")

        schedule = {
            "morningstart": "8:30",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "17:00",
            "comment": "",
        }
        with freeze_time("2021-09-14 8:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertIn("Open at", result)
        with freeze_time("2021-09-14 12:20:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Open")
        with freeze_time("2021-09-14 17:00:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Closed")

        schedule = {
            "morningstart": "8:30",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "17:00",
            "comment": "Full day opening!",
        }
        with freeze_time("2021-09-14 12:20:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Open (Full day opening!)")

        schedule = {
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "",
            "comment": "",
        }
        with freeze_time("2021-09-14 12:20:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Closed")

        schedule = {
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "",
            "comment": "It's closed!",
        }
        with freeze_time("2021-09-14 12:20:00"):
            result = get_schedule_for_today(schedule)
            self.assertEqual(result, "Closed (It's closed!)")

    def test_formatted_schedule(self):
        schedule = {
            "morningstart": "08:30",
            "morningend": "12:00",
            "afternoonstart": "13:00",
            "afternoonend": "17:00",
            "comment": "",
        }
        self.assertEqual("08:30 - 12:00 | 13:00 - 17:00", formatted_schedule(schedule))

        schedule = {
            "morningstart": "08:30",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "17:00",
            "comment": "",
        }
        self.assertEqual("08:30 - 17:00", formatted_schedule(schedule))

        schedule = {
            "morningstart": "08:30",
            "morningend": "12:00",
            "afternoonstart": "",
            "afternoonend": "",
            "comment": "",
        }
        self.assertEqual("08:30 - 12:00", formatted_schedule(schedule))

        schedule = {
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "",
            "comment": "",
        }
        self.assertEqual("Closed", formatted_schedule(schedule))

        schedule = {
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "13:00",
            "afternoonend": "17:00",
            "comment": "",
        }
        self.assertEqual("13:00 - 17:00", formatted_schedule(schedule))

        schedule = {
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "13:00",
            "afternoonend": "17:00",
            "comment": "Opening only on PM",
        }
        self.assertEqual(
            "13:00 - 17:00 (Opening only on PM)", formatted_schedule(schedule)
        )

    @requests_mock.Mocker()
    def test_formatted_with_multi_schedule(self, m):
        contact = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionContact",
            title="My contact",
        )
        authentic_contact_uid = "2dc381f0fb584381b8e4a19c84f53b35"
        contact.related_contacts = [authentic_contact_uid]
        contact_search_url = (
            "http://localhost:8080/Plone/@search?UID={}&fullobjects=1".format(
                authentic_contact_uid
            )
        )
        m.get(contact_search_url, text=json.dumps(self.json_contact))
        json_contact = ContactProperties(self.json_contact.get("items")[0])
        with freeze_time("2021-06-30 12:20:00"):
            schedule = json_contact.get_opening_informations()
            self.assertEqual(
                "13:00 - 17:30 (Ouverture PM)",
                json_contact.formatted_schedule(schedule),
            )
        with freeze_time("2021-07-07 12:20:00"):
            schedule = json_contact.get_opening_informations()
            self.assertEqual(
                "13:00 - 15:00 (Ouverture PM vacances)",
                json_contact.formatted_schedule(schedule),
            )
        with freeze_time("2021-09-01 12:20:00"):
            schedule = json_contact.get_opening_informations()
            self.assertEqual(
                "13:00 - 17:30 (Ouverture PM)",
                json_contact.formatted_schedule(schedule),
            )
        with freeze_time("2021-12-29 12:20:00"):
            schedule = json_contact.get_opening_informations()
            self.assertEqual(
                "13:00 - 15:00 (Ouverture PM vacances)",
                json_contact.formatted_schedule(schedule),
            )

    @requests_mock.Mocker()
    def test_empty_schedule(self, m):
        json_contact_empty_schedule = get_json(
            "resources/json_contact_empty_schedule_raw_mock.json"
        )
        contact = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionContact",
            title="My contact",
        )
        contact.visible_blocks = ["titles", "gallery", "schedule"]
        authentic_contact_uid = "2dc381f0fb584381b8e4a19c84f53b35"
        contact.related_contacts = [authentic_contact_uid]
        contact_search_url = (
            "http://localhost:8080/Plone/@search?UID={}&fullobjects=1".format(
                authentic_contact_uid
            )
        )
        m.get(contact_search_url, text=json.dumps(json_contact_empty_schedule))
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        json_contact = ContactProperties(json_contact_empty_schedule.get("items")[0])
        is_empty = json_contact.is_empty_schedule()
        self.assertEqual(is_empty, True)
        self.assertNotIn('class="schedule"', view())
        json_contact_empty_schedule["items"][0].get("schedule")["monday"] = {
            "morningstart": "8:00",
            "morningend": "12:00",
            "afternoonstart": "",
            "afternoonend": "",
            "comments": "",
        }
        m.get(contact_search_url, text=json.dumps(json_contact_empty_schedule))
        json_contact = ContactProperties(json_contact_empty_schedule.get("items")[0])
        is_empty = json_contact.is_empty_schedule()
        self.assertEqual(is_empty, False)
        self.assertIn('class="schedule"', view())

    @requests_mock.Mocker()
    def test_leadimage_is_in_portrait_mode(self, m):
        contact = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionContact",
            title="My contact",
        )
        view = queryMultiAdapter((self.page, self.request), name="full_view")
        authentic_contact_uid = "2dc381f0fb584381b8e4a19c84f53b35"
        contact.related_contacts = [authentic_contact_uid]
        contact_search_url = (
            "http://localhost:8080/Plone/@search?UID={}&fullobjects=1".format(
                authentic_contact_uid
            )
        )
        contact.visible_blocks = ["titles", "leadimage"]
        m.get(contact_search_url, text=json.dumps(self.json_contact))
        self.assertNotIn("contact_leadimage portrait", view())
        contact.is_in_portrait_mode = True
        self.assertIn("contact_leadimage portrait", view())

    @requests_mock.Mocker()
    def test_contact_modified(self, m):
        contact = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionContact",
            title="My contact",
        )
        authentic_contact_uid = "2dc381f0fb584381b8e4a19c84f53b35"
        contact.related_contacts = [authentic_contact_uid]
        contact_search_url = (
            "http://localhost:8080/Plone/@search?UID={}&fullobjects=1".format(
                authentic_contact_uid
            )
        )
        m.get(contact_search_url, text=json.dumps(self.json_contact))
        contact_view = queryMultiAdapter((contact, self.request), name="view")

        annotations = IAnnotations(contact)
        self.assertIsNone(annotations.get(SECTION_ITEMS_HASH_KEY))

        self.assertIsNotNone(contact_view.contacts())
        hash_1 = annotations.get(SECTION_ITEMS_HASH_KEY)
        self.assertIsNotNone(hash_1)
        first_modification = self.page.ModificationDate()

        sleep(1)
        m.get(contact_search_url, text=json.dumps(self.json_no_contact))
        self.assertIsNone(contact_view.contacts())
        next_modification = self.page.ModificationDate()
        hash_2 = annotations.get(SECTION_ITEMS_HASH_KEY)
        self.assertNotEqual(hash_1, hash_2)
        self.assertNotEqual(first_modification, next_modification)

        sleep(1)
        self.assertIsNone(contact_view.contacts())
        last_modification = self.page.ModificationDate()
        hash_3 = annotations.get(SECTION_ITEMS_HASH_KEY)
        self.assertEqual(hash_2, hash_3)
        self.assertEqual(next_modification, last_modification)



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets import common
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory


class ProcedureViewlet(common.ViewletBase):
    def get_selected_procedure_title(self):
        if self.context.procedure_ts is None:
            return
        factory = getUtility(
            IVocabularyFactory, "imio.smartweb.vocabulary.PublikProcedures"
        )
        vocabulary = factory()
        term = vocabulary.getTerm(self.context.procedure_ts)
        return term

    @property
    def is_anonymous(self):
        return self.portal_state.anonymous()



# -*- coding: utf-8 -*-

from Acquisition import aq_parent
from imio.smartweb.core.utils import reindexParent


def added_section(obj, event):
    if obj.collapsible_section:
        obj.hide_title = False
    reindexParent(obj, event)


def modified_section(obj, event):
    if obj.collapsible_section:
        obj.hide_title = False
    reindexParent(obj, event)


def removed_section(obj, event):
    if event.newParent is None:
        return
    parent = aq_parent(obj)
    if parent is not None:
        # in some cases (ex: relation breaking), we do not get the object in
        # its acquisition chain
        parent.reindexObject()



# -*- coding: utf-8 -*-

from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.row import DictRow
from imio.smartweb.common.config import DESCRIPTION_MAX_LENGTH
from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import schema
from plone.autoform import directives
from zope.interface import Interface
from zope.interface import implementer


class IPostit(Interface):
    """Postit schema"""

    title = schema.TextLine(
        title=_("Title"),
        required=False,
        default="",
        missing_value="",
    )

    subtitle = schema.TextLine(
        title=_("Subtitle"),
        required=False,
        default="",
        missing_value="",
    )

    description = schema.Text(
        title=_("Description"),
        description=_(
            "Use **text** to set text in bold. Limited to ${max} characters.",
            mapping={"max": DESCRIPTION_MAX_LENGTH},
        ),
        max_length=DESCRIPTION_MAX_LENGTH,
        required=False,
        default="",
        missing_value="",
    )


class ISectionPostit(ISection):
    """Marker interface and Dexterity Python Schema for SectionPostit"""

    description = schema.Text(
        title=_("Description"),
        description=_(
            "Use **text** to set text in bold. Limited to ${max} characters.",
            mapping={"max": DESCRIPTION_MAX_LENGTH},
        ),
        max_length=DESCRIPTION_MAX_LENGTH,
        required=False,
    )

    directives.widget(
        "postits", DataGridFieldFactory, allow_reorder=True, auto_append=False
    )
    postits = schema.List(
        title=_("Post-its"),
        value_type=DictRow(title=_("Post-it"), schema=IPostit),
        min_length=1,
        missing_value=[],
    )

    nb_results_by_batch = schema.Choice(
        title=_("Number of items per batch"),
        required=True,
        default=3,
        values=range(1, 9),
    )


@implementer(ISectionPostit)
class SectionPostit(Section):
    """SectionPostit class"""

    show_items_description = True
    show_items_lead_image = False



# -*- coding: utf-8 -*-

from collective.instancebehavior.interfaces import IInstanceBehaviorAssignableContent
from imio.smartweb.core.contents import IDefaultPages
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.autoform import directives
from plone.dexterity.content import Container
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from plone.uuid.interfaces import IUUID
from z3c.form.browser.radio import RadioFieldWidget
from zope import schema
from zope.interface import implementer
from zope.interface import alsoProvides
from zope.interface import noLongerProvides


class IFolder(model.Schema):
    """Marker interface and Dexterity Python Schema for Folder"""

    directives.widget(default_page_uid=RadioFieldWidget)
    directives.mode(default_page_uid="hidden")
    default_page_uid = schema.Choice(
        title=_("Item(s) that can be selected as the folder default page."),
        description=_(
            "For the anonymous visitor, the folder and the selected item are one. "
            "When he clicks on the folder, the item is displayed."
        ),
        required=False,
        vocabulary="imio.smartweb.vocabulary.CurrentFolderPages",
    )

    model.fieldset(
        "layout",
        label=_("Layout"),
        fields=["banner"],
    )

    banner = NamedBlobImage(
        title=_("Banner"),
        description=_("Define a banner for folder and children"),
        required=False,
    )


@implementer(IFolder, IInstanceBehaviorAssignableContent)
class Folder(Container):
    """Folder class"""

    def canSetDefaultPage(self):
        return False

    def setLayout(self, layout):
        """Disable current default page when layout is changed on a folder"""
        super(Folder, self).setLayout(layout)
        current_default_page = self.get_default_item(object=True)
        if current_default_page is not None:
            noLongerProvides(current_default_page, IDefaultPages)
            current_default_page.exclude_from_nav = False
            current_default_page.reindexObject(
                idxs=("object_provides", "exclude_from_nav")
            )
            self.default_page_uid = None
        self.reindexObject()

    def get_default_item(self, object=False):
        if self.default_page_uid:
            brains = api.content.find(context=self, UID=self.default_page_uid)
            if brains:
                brain = brains[0]
                return object is True and brain.getObject() or brain

    def set_default_item(self, old_default_item=None, new_default_item=None):
        if old_default_item is not None:
            noLongerProvides(old_default_item, IDefaultPages)
            old_default_item.exclude_from_nav = False
            old_default_item.reindexObject(idxs=["object_provides", "exclude_from_nav"])
        if new_default_item is None:
            # New default item has already been set on folder
            new_default_item = self.get_default_item(object=True)
        else:
            # We need to set the new default item on folder
            uuid = IUUID(new_default_item)
            self.default_page_uid = uuid
        alsoProvides(new_default_item, IDefaultPages)
        new_default_item.exclude_from_nav = True
        new_default_item.reindexObject(idxs=["object_provides", "exclude_from_nav"])



# -*- coding: utf-8 -*-

from zope.annotation.interfaces import IAnnotations

import json
import os


# "imio.smartweb.SectionContact",
# "imio.smartweb.SectionEvents",
# "imio.smartweb.SectionNews",
# "imio.smartweb.SectionSendinblue",
# "imio.smartweb.SectionSlide",


def get_sections_types(kw="type"):
    sections = [
        {
            "type": "imio.smartweb.SectionFiles",
            "empty_section": True,
        },
        {
            "type": "imio.smartweb.SectionGallery",
            "empty_section": True,
        },
        {
            "type": "imio.smartweb.SectionHTML",
            "empty_section": True,
        },
        {
            "type": "imio.smartweb.SectionLinks",
            "empty_section": True,
        },
        {
            "type": "imio.smartweb.SectionMap",
            "empty_section": True,
        },
        {
            "type": "imio.smartweb.SectionPostit",
            "empty_section": True,
        },
        {
            "type": "imio.smartweb.SectionSelections",
            "empty_section": True,
        },
        {
            "type": "imio.smartweb.SectionText",
            "empty_section": True,
        },
        {
            "type": "imio.smartweb.SectionVideo",
            "empty_section": True,
        },
    ]
    if kw == "empty_section":
        return [
            section.get("type")
            for section in sections
            if section.get("empty_section") is True
        ]
    else:
        return [section.get("type") for section in sections]


def get_json(json_filename):
    with open(
        os.path.join(
            os.path.dirname(__file__),
            json_filename,
        ),
    ) as json_file:
        json_procedures_raw_mock = json.load(json_file)
        return json_procedures_raw_mock


def get_html(html_filename):
    with open(
        os.path.join(
            os.path.dirname(__file__),
            html_filename,
        ),
    ) as html_file:
        html_raw_mock = html_file.read()
        return html_raw_mock


def clear_cache(request):
    annotations = IAnnotations(request)
    del annotations["plone.memoize"]


def make_named_image(filename="plone.png"):
    path = os.path.join(os.path.dirname(__file__), f"resources/{filename}")
    with open(path, "rb") as f:
        image_data = f.read()
    return {"filename": filename, "data": image_data}



# -*- coding: utf-8 -*-

from imio.smartweb.common.config import DESCRIPTION_MAX_LENGTH
from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform.directives import write_permission
from zope import schema
from zope.interface import implementer


class ISectionHTML(ISection):
    """Marker interface and Dexterity Python Schema for SectionHTML"""

    write_permission(html="imio.smartweb.core.CanManageSectionHTML")
    html = schema.SourceText(
        title=_("HTML"),
        description=_(
            "Enter your HTML code directly here. It must be nested in tags such as: "
            "<code>&lt;p&gt;code&lt;/p&gt;</code>, "
            "<code>&lt;span&gt;code&lt;/span&gt;</code>, "
            "<code>&lt;div&gt;code&lt;/div&gt;</code>"
        ),
        required=True,
    )

    description = schema.Text(
        title=_("Description"),
        description=_(
            "Use **text** to set text in bold. Limited to ${max} characters.",
            mapping={"max": DESCRIPTION_MAX_LENGTH},
        ),
        max_length=DESCRIPTION_MAX_LENGTH,
        required=False,
    )


@implementer(ISectionHTML)
class SectionHTML(Section):
    """SectionHTML class"""

    manage_content = True



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from zope.interface import implementer


class ISectionMap(ISection):
    """Marker interface and Dexterity Python Schema for SectionMap"""


@implementer(ISectionMap)
class SectionMap(Section):
    """SectionMap class"""



# -*- coding: utf-8 -*-

from imio.smartweb.common.caching import ban_physicalpath
from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents import Pages
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform.directives import write_permission
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from Products.CMFPlone.utils import parent
from zope import schema
from zope.globalrequest import getRequest
from zope.interface import implementer


class IFooter(IPages):
    """Marker interface and Dexterity Python Schema for Footer"""

    model.fieldset(
        "layout", label=_("Layout"), fields=["background_image", "css_class"]
    )

    write_permission(background_image="cmf.ManagePortal")
    background_image = NamedBlobImage(
        title=_("Background image"),
        required=False,
    )

    write_permission(css_class="cmf.ManagePortal")
    css_class = schema.TextLine(title=_("CSS class"), default="", required=False)


@implementer(IFooter)
class Footer(Pages):
    """Footer class"""

    def notifyModified(self):
        super(Footer, self).notifyModified()
        request = getRequest()
        container = parent(self)
        if not container:
            return
        physical_path = container.getPhysicalPath()
        ban_physicalpath(request, physical_path)



# -*- coding: utf-8 -*-

from eea.facetednavigation.layout.interfaces import IFacetedLayout
from freezegun import freeze_time
from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.namedfile.file import NamedBlobImage
from plone.uuid.interfaces import IUUID
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides


class TestFaceted(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.collection = api.content.create(
            container=self.portal,
            type="Collection",
            title="A collection",
            id="a-collection",
        )

    def test_faceted_body_class(self):
        template = self.collection.restrictedTraverse("view")
        view = getMultiAdapter((self.collection, self.request), name="view")
        layout_view = self.collection.restrictedTraverse("@@plone_layout")
        body_class = layout_view.bodyClass(template, view)
        self.assertIn("faceted-block-view", body_class)

        IFacetedLayout(self.collection).update_layout(
            "faceted-summary-view-with-images"
        )
        body_class = layout_view.bodyClass(template, view)
        self.assertIn("faceted-summary-view-with-images", body_class)

        IFacetedLayout(self.collection).update_layout("faceted-map")
        body_class = layout_view.bodyClass(template, view)
        self.assertIn("faceted-map", body_class)

        subtyper = self.collection.restrictedTraverse("@@faceted_subtyper")
        subtyper.disable()
        view = getMultiAdapter((self.collection, self.request), name="view")
        body_class = layout_view.bodyClass(template, view)
        self.assertNotIn("faceted-summary-view-with-images", body_class)

    @freeze_time("2021-09-14 8:00:00")
    def test_map_popup(self):
        alsoProvides(self.request, IImioSmartwebCoreLayer)
        collection = api.content.create(
            container=self.portal,
            type="Collection",
            title="My collection",
            description="My **description**",
        )
        uuid = IUUID(collection)
        brain = api.content.find(UID=uuid)[0]
        popup_view = getMultiAdapter(
            (collection, self.request), name="faceted-map-geojson-popup"
        )
        popup = popup_view.popup(brain)
        self.assertNotIn("<img", popup)
        self.assertIn('href="http://nohost/plone/my-collection"', popup)
        self.assertIn("My collection", popup)
        self.assertIn("My description", popup)

        collection.image = NamedBlobImage(**make_named_image())
        collection.reindexObject()
        brain = api.content.find(UID=uuid)[0]
        popup = popup_view.popup(brain)
        self.assertIn(
            '<img src="http://nohost/plone/my-collection/@@images/image/mini?cache_key=78fd1bab198354b6877aed44e2ea0b4d',
            popup,
        )

    @freeze_time("2021-09-14 8:00:00")
    def test_get_scale_url(self):
        collection = api.content.create(
            container=self.portal,
            type="Collection",
            title="My collection",
        )
        faceted_view = queryMultiAdapter(
            (collection, self.request), name="faceted-view"
        )

        # page with no lead image
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        uuid = IUUID(page)
        brain = api.content.find(UID=uuid)[0]
        self.assertEqual(faceted_view.get_scale_url(brain), "")

        # page with lead image
        page.image = NamedBlobImage(**make_named_image())
        page.reindexObject()
        brain = api.content.find(UID=uuid)[0]
        self.assertEqual(
            faceted_view.get_scale_url(brain),
            "http://nohost/plone/page/@@images/image/vignette?cache_key=78fd1bab198354b6877aed44e2ea0b4d",
        )

        # empty gallery
        gallery = api.content.create(
            container=page,
            type="imio.smartweb.SectionGallery",
            title="Gallery",
        )
        uuid = IUUID(gallery)
        brain = api.content.find(UID=uuid)[0]
        self.assertEqual(faceted_view.get_scale_url(brain), "")

        # gallery with image
        image = api.content.create(
            container=gallery,
            type="Image",
            title="Image",
        )
        image.image = NamedBlobImage(**make_named_image())
        brain = api.content.find(UID=uuid)[0]
        faceted_view.get_scale_url(brain)
        scale_url = faceted_view.get_scale_url(brain)
        self.assertNotIn(
            "http://nohost/plone/page/gallery/image/@@images/image/vignette", scale_url
        )
        self.assertIn(
            "http://nohost/plone/page/gallery/image/@@images/image-390-", scale_url
        )



# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup
from imio.smartweb.core.contents import IDefaultPages
from imio.smartweb.core.contents.folder.views import ElementView
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.utils import get_default_content_id
from plone import api
from plone.app.testing import logout
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import setRoles
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.testing.zope import Browser
from plone.uuid.interfaces import IUUID
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.publisher.browser import TestRequest
from zope.schema.interfaces import IVocabularyFactory
import transaction


class TestDefaultPages(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        self.defaultpage = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Default Page",
            id="defaultpage",
        )
        uuid = IUUID(self.defaultpage)
        self.folder.setLayout("element_view")
        request = TestRequest(
            form={"form.widgets.default_page_uid": uuid, "form.buttons.apply": "Apply"}
        )
        alsoProvides(request, IPloneFormLayer)
        form = ElementView(self.folder, request)
        form.update()

    def test_no_change(self):
        api.content.create(
            container=self.folder, type="imio.smartweb.Page", title="Page 1", id="page1"
        )
        self.folder.setLayout("element_view")
        request = TestRequest(form={"form.buttons.apply": "Apply"})
        alsoProvides(request, IPloneFormLayer)
        form = ElementView(self.folder, request)
        form.update()
        data, errors = form.extractData()
        self.assertTrue(len(errors) == 0)
        self.assertEqual(form.status, "No changes were applied.")

    def test_exclude_from_nav(self):
        page1 = api.content.create(
            container=self.folder, type="imio.smartweb.Page", title="Page 1", id="page1"
        )
        page2 = api.content.create(
            container=self.folder, type="imio.smartweb.Page", title="Page 2", id="page2"
        )
        uuid1 = IUUID(page1)
        uuid2 = IUUID(page2)
        self.folder.setLayout("element_view")
        request = TestRequest(
            form={"form.widgets.default_page_uid": uuid1, "form.buttons.apply": "Apply"}
        )
        alsoProvides(request, IPloneFormLayer)
        form = ElementView(self.folder, request)
        form.update()
        data, errors = form.extractData()
        self.assertTrue(len(errors) == 0)
        self.assertTrue(page1.exclude_from_nav)
        self.assertTrue(IDefaultPages.providedBy(page1))
        self.assertFalse(page2.exclude_from_nav)
        self.assertFalse(IDefaultPages.providedBy(page2))

        request = TestRequest(
            form={"form.widgets.default_page_uid": uuid2, "form.buttons.apply": "Apply"}
        )
        alsoProvides(request, IPloneFormLayer)
        form = ElementView(self.folder, request)
        form.update()
        data, errors = form.extractData()
        self.assertTrue(len(errors) == 0)
        self.assertTrue(page2.exclude_from_nav)
        self.assertTrue(IDefaultPages.providedBy(page2))
        self.assertFalse(page1.exclude_from_nav)
        self.assertFalse(IDefaultPages.providedBy(page1))

        self.folder.setLayout("block_view")
        self.assertIsNone(self.folder.default_page_uid)
        self.assertFalse(page1.exclude_from_nav)
        self.assertFalse(IDefaultPages.providedBy(page1))
        self.assertFalse(page2.exclude_from_nav)
        self.assertFalse(IDefaultPages.providedBy(page2))

    def test_exclude_from_nav_field(self):
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/edit".format(self.defaultpage.absolute_url()))
        content = browser.contents
        soup = BeautifulSoup(content)
        exclude_from_nav_widget = soup.find(
            id="form-widgets-IExcludeFromNavigation-exclude_from_nav"
        )
        self.assertIsNone(exclude_from_nav_widget.find("input"))
        children = [
            c for c in exclude_from_nav_widget if not isinstance(c, str) or c.strip()
        ]
        self.assertEqual(len(children), 1)
        self.assertEqual(children[0].text, "yes")

        page2 = api.content.create(
            container=self.folder, type="imio.smartweb.Page", title="Page 2", id="page2"
        )
        uuid2 = IUUID(page2)
        request = TestRequest(
            form={"form.widgets.default_page_uid": uuid2, "form.buttons.apply": "Apply"}
        )
        alsoProvides(request, IPloneFormLayer)
        form = ElementView(self.folder, request)
        form.update()
        data, errors = form.extractData()
        transaction.commit()

        browser.open("{}/edit".format(self.defaultpage.absolute_url()))
        content = browser.contents
        soup = BeautifulSoup(content)
        exclude_from_nav_widget = soup.find(
            id="form-widgets-IExcludeFromNavigation-exclude_from_nav"
        )
        self.assertIsNotNone(exclude_from_nav_widget.find("input"))

    def test_breadcrumbs(self):
        page1 = api.content.create(
            container=self.folder, type="imio.smartweb.Page", title="Page 1", id="page1"
        )
        self.folder.setLayout("element_view")

        breadcrumbs_view = getMultiAdapter(
            (page1, self.request), name="breadcrumbs_view"
        )
        breadcrumbs = breadcrumbs_view.breadcrumbs()
        self.assertEqual(len(breadcrumbs), 2)
        self.assertEqual(breadcrumbs[-1]["Title"], "Page 1")

        self.folder.set_default_item(
            old_default_item=self.defaultpage, new_default_item=page1
        )
        breadcrumbs_view = getMultiAdapter(
            (page1, self.request), name="breadcrumbs_view"
        )
        breadcrumbs = breadcrumbs_view.breadcrumbs()
        self.assertEqual(len(breadcrumbs), 2)
        self.assertEqual(breadcrumbs[-1]["Title"], "Page 1")

        # Anonymus users should not see default pages in breadcrumbs
        logout()
        breadcrumbs_view = getMultiAdapter(
            (page1, self.request), name="breadcrumbs_view"
        )
        breadcrumbs = breadcrumbs_view.breadcrumbs()
        self.assertEqual(len(breadcrumbs), 1)
        self.assertEqual(breadcrumbs[-1]["Title"], "Folder")

    def test_page_anonymous(self):
        page = api.content.create(
            container=self.folder, type="imio.smartweb.Page", title="Page"
        )
        self.folder.setLayout("element_view")
        self.folder.set_default_item(
            old_default_item=self.defaultpage, new_default_item=page
        )
        logout()
        view = getMultiAdapter((page, self.request), name="view")
        self.assertIn("<h1>Page</h1>", view())

    def test_collection_anonymous(self):
        collection = api.content.create(
            container=self.folder, type="Collection", title="Collection"
        )
        self.folder.setLayout("element_view")
        self.folder.set_default_item(
            old_default_item=self.defaultpage, new_default_item=collection
        )
        logout()
        view = getMultiAdapter((collection, self.request), name="view")()
        self.assertIn("<h1>Collection</h1>", view)
        self.assertIn("faceted-block-view", view)

    def test_default_page_removal(self):
        self.assertTrue(self.defaultpage.exclude_from_nav)
        self.assertTrue(IDefaultPages.providedBy(self.defaultpage))
        api.content.delete(self.defaultpage)
        self.assertIsNone(self.folder.default_page_uid)

    def test_default_page_rename(self):
        api.content.rename(obj=self.defaultpage, new_id="foo")
        self.assertTrue(self.defaultpage.exclude_from_nav)
        self.assertTrue(IDefaultPages.providedBy(self.defaultpage))
        self.assertEqual(self.folder.default_page_uid, IUUID(self.defaultpage))

    def test_default_page_cut_and_paste_in_a_folder(self):
        folder2 = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder2",
            id="folder2",
        )
        api.content.move(source=self.defaultpage, target=folder2)
        self.assertFalse(self.defaultpage.exclude_from_nav)
        self.assertFalse(IDefaultPages.providedBy(self.defaultpage))
        self.assertIsNone(self.folder.default_page_uid)

    def test_default_page_cut_and_paste_anywhere(self):
        api.content.move(source=self.defaultpage, target=self.portal)
        self.assertFalse(self.defaultpage.exclude_from_nav)
        self.assertFalse(IDefaultPages.providedBy(self.defaultpage))
        self.assertIsNone(self.folder.default_page_uid)

    def test_default_page_copy_and_paste(self):
        copied_page = api.content.copy(source=self.defaultpage, target=self.folder)
        self.assertTrue(self.defaultpage.exclude_from_nav)
        self.assertTrue(IDefaultPages.providedBy(self.defaultpage))
        self.assertEqual(self.folder.default_page_uid, IUUID(self.defaultpage))
        self.assertFalse(copied_page.exclude_from_nav)
        self.assertFalse(IDefaultPages.providedBy(copied_page))

        folder2 = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder2",
            id="folder2",
        )
        copied_page = api.content.copy(source=self.defaultpage, target=folder2)
        self.assertTrue(self.defaultpage.exclude_from_nav)
        self.assertTrue(IDefaultPages.providedBy(self.defaultpage))
        self.assertEqual(self.folder.default_page_uid, IUUID(self.defaultpage))
        self.assertFalse(copied_page.exclude_from_nav)
        self.assertFalse(IDefaultPages.providedBy(copied_page))

    def test_eligible_default_contents(self):
        factory = getUtility(
            IVocabularyFactory, "imio.smartweb.vocabulary.CurrentFolderPages"
        )
        vocabulary = factory(self.folder)
        self.assertEqual(len(vocabulary), 1)

        footer_settings = getMultiAdapter(
            (self.folder, self.request), name="footer_settings"
        )
        footer_settings.add_footer()
        vocabulary = factory(self.folder)
        self.assertEqual(len(vocabulary), 1)

        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        herobanner_settings = getMultiAdapter(
            (self.folder, self.request), name="herobanner_settings"
        )
        herobanner_settings.add_herobanner()
        vocabulary = factory(self.folder)
        self.assertEqual(len(vocabulary), 1)

        api.content.create(
            container=self.folder,
            type="Collection",
            title="Collection",
        )
        vocabulary = factory(self.folder)
        self.assertEqual(len(vocabulary), 2)

    def test_get_default_content_id(self):
        self.assertEqual(get_default_content_id(self.folder), "defaultpage")
        folder_without_default = api.content.create(
            container=self.portal, type="imio.smartweb.Folder", title="Folder"
        )
        self.assertEqual(get_default_content_id(folder_without_default), "")



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import queryMultiAdapter


class TestSectionSendInBlue(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portalpage = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            id="Portal page",
        )

    def test_button_position(self):
        section_sendinblue = api.content.create(
            container=self.portalpage,
            type="imio.smartweb.SectionSendinblue",
            title="SendInBlue",
        )
        section_view = queryMultiAdapter(
            (section_sendinblue, self.request), name="view"
        )
        self.assertEqual(section_view.button_position, "button_bottom")

        portal_page_view = queryMultiAdapter(
            (self.portalpage, self.request), name="full_view"
        )
        self.assertIn("button_bottom", portal_page_view())

        api.portal.set_registry_record(
            "smartweb.sendinblue_button_position", "button_right"
        )
        portal_page_view = queryMultiAdapter(
            (self.portalpage, self.request), name="full_view"
        )
        self.assertIn("button_right", portal_page_view())

    def test_button_text(self):
        section_sendinblue = api.content.create(
            container=self.portalpage,
            type="imio.smartweb.SectionSendinblue",
            title="SendInBlue",
        )
        section_view = queryMultiAdapter(
            (section_sendinblue, self.request), name="view"
        )
        self.assertEqual(section_view.button_text, "Subscribe")

        portal_page_view = queryMultiAdapter(
            (self.portalpage, self.request), name="full_view"
        )
        self.assertIn('value="Subscribe"', portal_page_view())

        texts = [
            {"language": "fr", "text": "Je m'inscris!"},
            {"language": "en", "text": "Register!"},
        ]
        api.portal.set_registry_record("smartweb.sendinblue_button_text", texts)
        portal_page_view = queryMultiAdapter(
            (self.portalpage, self.request), name="full_view"
        )
        self.assertNotIn('value="Subscribe"', portal_page_view())
        self.assertIn('value="Register!"', portal_page_view())



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.supermodel import model
from zope import schema
from zope.interface import implementer


class ISectionFiles(ISection):
    """Marker interface and Dexterity Python Schema for SectionFiles"""

    nb_results_by_batch = schema.Choice(
        title=_("Number of items per batch"),
        required=True,
        default=1,
        values=[1, 3, 4],
    )

    model.fieldset("layout", fields=["show_items_lead_image"])
    show_items_lead_image = schema.Bool(
        title=_("Show items lead image"),
        required=False,
    )


@implementer(ISectionFiles)
class SectionFiles(Section):
    """SectionText class"""

    manage_content = True



# -*- coding: utf-8 -*-

from plone import api
from Products.CMFPlone.browser.search import Search
from Products.CMFPlone.utils import normalizeString

import json


class Search(Search):
    valid_keys = ("sort_on", "sort_order", "sort_limit", "fq", "fl", "facet", "core")

    def results(self):
        self._core = None
        self._query_parameters = {}
        return super(Search, self).results()

    def results_news(self):
        self._core = "news"
        self._query_parameters = {"portal_type": ["imio.news.NewsItem"]}
        return super(Search, self).results()

    def filter_query(self, query):
        query = super(Search, self).filter_query(query)
        if self._core is not None:
            query["core"] = self._core
        if self._query_parameters:
            query.update(self._query_parameters)
        return query

    def get_item_url(self, item):
        """Return a local url depending of current website config for remote items"""
        mapping = {
            "imio.news.NewsItem": "imio.smartweb.NewsView",
        }
        view_type = mapping.get(item.portal_type)
        if not view_type:
            # This should never happen
            raise NotImplementedError
        content_view = api.content.find(
            context=api.portal.get_navigation_root(item),
            portal_type=view_type,
        )
        if not content_view:
            # This mean that there is no content listing
            raise ValueError("Missing content type {0}".format(view_type))
        # XXX Use informations from content view and item to determinate the right
        # context
        item_id = normalizeString(item.Title)
        return "{container}/{id}?u={uid}".format(
            container=content_view[0].getURL(),
            id=item_id,
            uid=item.UID,
        )

    def get_search_result_option(self):
        directory_entity_uid = api.portal.get_registry_record(
            "smartweb.directory_entity_uid"
        )
        events_entity_uid = api.portal.get_registry_record("smartweb.events_entity_uid")
        news_entity_uid = api.portal.get_registry_record("smartweb.news_entity_uid")

        response = {
            "directory": bool(directory_entity_uid),
            "events": bool(events_entity_uid),
            "news": bool(news_entity_uid),
        }

        return json.dumps(response)



from Products.Five.browser import BrowserView
from imio.smartweb.core.browser.utils import get_plausible_vars


class PlausibleView(BrowserView):
    @property
    def is_plausible_set(self):
        return True if get_plausible_vars(self) else False

    @property
    def get_embedhostjs_src(self):
        vars = get_plausible_vars(self)
        return f"https://{vars['plausible_url']}/js/embed.host.js"

    @property
    def get_iframe_src(self):
        vars = get_plausible_vars(self)
        return f"https://{vars['plausible_url']}/share/{vars['plausible_site']}?auth={vars['plausible_token']}&embed=true&theme=light&background=transparent"



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import ICropping
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_sections_types
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.namedfile.file import NamedBlobImage
from zope.component import getMultiAdapter


class TestCropping(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        self.page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Default Page",
            id="defaultpage",
        )

    def test_cropping_adapter(self):
        # footer cropping
        view = getMultiAdapter((self.portal, self.request), name="footer_settings")
        view.add_footer()
        footer = self.portal.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )[0]
        adapter = ICropping(footer, alternate=None)
        self.assertEqual([], adapter.get_scales("background_image", self.request))

        # minisite cropping
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        adapter = ICropping(self.folder, alternate=None)
        self.assertEqual(
            ["liste", "vignette", "slide"], adapter.get_scales("image", self.request)
        )
        self.assertEqual([], adapter.get_scales("banner", self.request))
        self.assertEqual(["preview"], adapter.get_scales("logo", self.request))
        view.disable()

        # subsite cropping
        view = getMultiAdapter((self.folder, self.request), name="subsite_settings")
        view.enable()
        adapter = ICropping(self.folder, alternate=None)
        self.assertEqual(
            ["liste", "vignette", "slide"], adapter.get_scales("image", self.request)
        )
        self.assertEqual([], adapter.get_scales("banner", self.request))
        self.assertEqual(["preview"], adapter.get_scales("logo", self.request))
        view.disable()

        # folder cropping
        adapter = ICropping(self.folder, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertEqual([], adapter.get_scales("banner", self.request))
        self.assertNotIn("banner", adapter.get_scales("image", self.request))

        # page cropping
        adapter = ICropping(self.page, alternate=None)
        self.assertEqual(
            ["liste", "vignette", "slide"], adapter.get_scales("image", self.request)
        )

        # sections cropping
        section_types = get_sections_types()
        for section_type in section_types:
            section = api.content.create(
                container=self.page,
                type=section_type,
                title="Title of my {}".format(section_type),
            )
            adapter = ICropping(section, alternate=None)
            self.assertIsNotNone(adapter)
            self.assertEqual([], adapter.get_scales("background_image", self.request))
            self.assertNotIn("banner", adapter.get_scales("image", self.request))

    def test_uncroppable_fields(self):
        self.folder.banner = NamedBlobImage(**make_named_image("plone.png"))
        self.folder.image = NamedBlobImage(**make_named_image("plone.png"))
        adapter = IImageCroppingUtils(self.folder, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertEqual(len(list(adapter._image_field_values())), 1)
        self.assertEqual(adapter.image_field_names(), ["image"])

    def test_cropping_view(self):
        self.folder.banner = NamedBlobImage(**make_named_image("plone.png"))
        self.folder.image = NamedBlobImage(**make_named_image("plone.png"))
        cropping_view = getMultiAdapter(
            (self.folder, self.request), name="croppingeditor"
        )
        self.assertEqual(len(list(cropping_view._scales("banner"))), 0)
        self.assertEqual(len(list(cropping_view._scales("image"))), 3)
        self.assertNotIn("Banner", cropping_view())
        self.assertIn("Lead Image", cropping_view())



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.views import CarouselOrTableSectionView
from imio.smartweb.core.utils import batch_results
from imio.smartweb.core.utils import get_scale_url

import logging

logger = logging.getLogger("imio.smartweb.core")


class SelectionsView(CarouselOrTableSectionView):
    """Selection Section view"""

    def items(self):
        image_scale = self.image_scale
        items = [rel.to_object for rel in self.context.selected_items]
        results = []
        for item in items:
            if item is None:
                # TODO we need to understand why relations break and fix that
                logger.warning(
                    f"Broken selection item in {self.context.absolute_url()}"
                )
                continue
            url = item.absolute_url()
            description = item.description or ""
            has_image = True if getattr(item.aq_base, "image", None) else False
            scale_url = get_scale_url(item, self.request, "image", image_scale)
            results.append(
                {
                    "title": item.title,
                    "description": description.replace("**", ""),
                    "url": url,
                    "image": scale_url,
                    "has_image": has_image,
                }
            )
        return batch_results(results, self.context.nb_results_by_batch)



# -*- coding: utf-8 -*-

from collective.instancebehavior import disable_behaviors
from collective.instancebehavior import enable_behaviors
from imio.smartweb.core.behaviors.subsite import IImioSmartwebSubsite
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.contents import IFolder
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from Products.Five.browser import BrowserView


class SubSiteSettings(BrowserView):
    """Subsite settings"""

    def enable(self):
        if not self.available():
            return
        enable_behaviors(
            self.context,
            [IImioSmartwebSubsite.__identifier__],
            [IImioSmartwebSubsite],
        )
        api.portal.show_message(
            _("Subsite has been successfully activated"), self.request
        )
        self.request.response.redirect(self.context.absolute_url())

    def disable(self):
        disable_behaviors(
            self.context,
            [IImioSmartwebSubsite.__identifier__],
            [IImioSmartwebSubsite],
        )
        api.portal.show_message(_("Subsite has been disabled"), self.request)
        self.request.response.redirect(self.context.absolute_url())

    def available(self):
        if IImioSmartwebMinisite.providedBy(self.context):
            return False
        return IFolder.providedBy(self.context) and not self.enabled()

    def enabled(self):
        if IImioSmartwebMinisite.providedBy(self.context):
            return False
        return IImioSmartwebSubsite.providedBy(self.context)



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import ILocalManagerAware
from imio.smartweb.core.contents import IFolder
from imio.smartweb.core.contents.cropping import SmartwebCroppingProvider
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.autoform.directives import write_permission
from plone.autoform.interfaces import IFormFieldProvider
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


class MinisiteCroppingProvider(SmartwebCroppingProvider):
    def get_scales(self, fieldname, request=None):
        if fieldname == "logo":
            # scale used for logo field
            return ["preview"]
        else:
            return super(MinisiteCroppingProvider, self).get_scales(fieldname, request)


class IImioSmartwebMinisite(INavigationRoot, ILocalManagerAware):
    """Marker interface to use minisite as a new navigation root"""


@provider(IFormFieldProvider)
class IImioSmartwebMinisiteSettings(model.Schema):
    model.fieldset(
        "layout",
        label=_("Layout"),
        fields=["logo", "logo_display_mode"],
    )

    write_permission(logo="imio.smartweb.core.CanEditMinisiteLogo")
    logo = NamedBlobImage(
        title=_("Logo"),
        description=_("Define a logo for minisite"),
        required=False,
    )

    logo_display_mode = schema.Choice(
        title=_("Logo display"),
        description=_("Choose your display mode"),
        source="imio.smartweb.vocabulary.SubsiteDisplayMode",
        required=True,
        default="title",
    )


@implementer(IImioSmartwebMinisiteSettings)
@adapter(IFolder)
class Minisite(object):
    """ """



# -*- coding: utf-8 -*-

from collective.instancebehavior import disable_behaviors
from collective.instancebehavior import enable_behaviors
from collective.instancebehavior import instance_behaviors_of
from imio.smartweb.common.utils import get_vocabulary
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.autoform import directives
from plone.autoform.form import AutoExtensibleForm
from plone.dexterity.content import ASSIGNABLE_CACHE_KEY
from plone.supermodel import model
from z3c.form import button
from z3c.form import form
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema


class IInstanceBehaviors(model.Schema):
    directives.widget(instance_behaviors=CheckBoxFieldWidget)
    instance_behaviors = schema.List(
        title=_("Available taxonomies"),
        description=_(
            "Changes will only affect this content. They must be carried out with full knowledge. (No test)"
        ),
        value_type=schema.Choice(
            vocabulary="imio.smartweb.vocabulary.AvailableInstanceBehaviors"
        ),
    )


class InstanceBehaviors(AutoExtensibleForm, form.Form):
    schema = IInstanceBehaviors
    ignoreContext = True
    enable_autofocus = False
    label = _("Taxonomies choices form")
    description = _(
        "Be careful, adding or removing taxonomies must be a part of strategy implemented in collaboration with iMio"
    )

    def updateWidgets(self):
        super(InstanceBehaviors, self).updateWidgets()
        instance_behaviors_voc = get_vocabulary(
            "imio.smartweb.vocabulary.AvailableInstanceBehaviors"
        )
        instance_behaviors = [
            voc.value
            for voc in instance_behaviors_voc
            if voc.value in instance_behaviors_of(self.context)
        ]
        self.widgets["instance_behaviors"].value = instance_behaviors

    @button.buttonAndHandler("Ok")
    def handleApply(self, action):
        data, errors = self.extractData()
        selected_instance_behaviors = data.get("instance_behaviors")
        enable_behaviors(self.context, selected_instance_behaviors, [])
        instance_behaviors_voc = get_vocabulary(
            "imio.smartweb.vocabulary.AvailableInstanceBehaviors"
        )
        disable_instance_behaviors = [
            voc.value
            for voc in instance_behaviors_voc
            if voc.value not in selected_instance_behaviors
        ]
        disable_behaviors(self.context, disable_instance_behaviors, [])
        if hasattr(self.request, ASSIGNABLE_CACHE_KEY):
            delattr(self.request, ASSIGNABLE_CACHE_KEY)
        if errors:
            self.status = self.formErrorsMessage
            return
        api.portal.show_message(
            _("Taxonomies are correctly set on your object!"), self.request
        )
        self.request.response.redirect(self.context.absolute_url())

    @button.buttonAndHandler("Cancel")
    def handleCancel(self, action):
        self.request.response.redirect(self.context.absolute_url())



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.rest.view import BaseRestView
from plone import api


class DirectoryViewView(BaseRestView):
    """DirectoryView view"""

    @property
    def propose_url(self):
        return api.portal.get_registry_record("smartweb.propose_directory_url")



# -*- coding: utf-8 -*-

from plone.dexterity.content import Container
from zope.interface import Interface
from zope.interface import implementer


class IPages(Interface):
    """Shared interface and schema for Pages"""


class IDefaultPages(Interface):
    """Marker interface for default Pages (used in Element view)"""


@implementer(IPages)
class Pages(Container):
    """Shared base class for Pages"""

    def canSetDefaultPage(self):
        return False
